@@ -178,13 +178,18 @@ def try_imports(imports, fail_msg=None):
 
     Returns
     -------
-    locals : dict
+    imported : dict
         The dictionary of imported modules.
     """
     msgs = []
+    imported = {}
     for imp in imports:
         try:
-            exec(imp)
+            if (sys.version_info.major, sys.version_info.minor) > (3, 12):
+                exec(imp, locals=imported)
+            else:
+                exec(imp)
+                imported = locals()
             break
 
         except Exception as inst:
@@ -195,7 +200,7 @@ def try_imports(imports, fail_msg=None):
             msgs.append(fail_msg)
             raise ValueError('\n'.join(msgs))
 
-    return locals()
+    return imported
 
 def python_shell(frame=0):
     import code
@@ -508,6 +513,9 @@ class IndexedStruct(Struct):
     def __setitem__(self, key, val):
         setattr(self, key, val)
 
+    def setdefault(self, key, default=None):
+        return self.set_default(key, default=default)
+
 ##
 # 14.07.2006, c
 class Container(Struct):
@@ -635,7 +643,7 @@ class Container(Struct):
         return out
 
     def remove_name(self, name):
-        ii = self.names.index[name]
+        ii = self.names.index(name)
         del self.names[ii]
         del self._objs[ii]
 
@@ -1046,7 +1054,10 @@ def is_integer(var):
 ##
 # 23.01.2006, c
 def is_sequence(var):
-    from collections import Sequence
+    try:
+        from collections.abc import Sequence
+    except ImportError:
+        from collections import Sequence
     if isinstance(var, basestr):
         return False
     return isinstance(var, Sequence)
