@@ -197,7 +197,7 @@ def transform_fields(adict):
         if isinstance(conf, tuple):
             c2 = tuple_to_conf(key, conf,
                                ['dtype', 'shape', 'region', 'approx_order',
-                                'space', 'poly_space_base'])
+                                'space', 'poly_space_basis'])
             if c2.dtype in dtypes:
                 c2.dtype = dtypes[c2.dtype]
             d2['field_%s__%d' % (c2.name, ii)] = c2
@@ -556,10 +556,6 @@ class ProblemConf(Struct):
             fun = None
 
         elif callable(name):
-            import inspect
-            if not (inspect.isfunction(name) or inspect.ismethod(name)):
-                msg = '`name` has to have `str` or `function` type! (got %s)'
-                raise ValueError(msg % type(name))
             fun = name
 
         else:
