@@ -2,9 +2,11 @@ from __future__ import absolute_import
 
 import numpy as nm
 
-from sfepy.base.base import output, iter_dict_of_lists, Struct, basestr
+from sfepy.base.base import output, iter_dict_of_lists, Struct, basestr,\
+    assert_
 from sfepy.base.timing import Timer
 import six
+from sfepy.mechanics.tensors import get_cauchy_strain
 
 
 def parse_approx_order(approx_order):
@@ -67,7 +69,7 @@ def setup_extra_data(conn_info):
         for var in info.all_vars:
             field = var.get_field()
             if var == info.primary:
-                field.setup_extra_data(info.ps_tg, info, info.is_trace)
+                field.setup_extra_data(info)
 
 def fields_from_conf(conf, regions):
     fields = {}
@@ -85,7 +87,7 @@ class Field(Struct):
 
     @staticmethod
     def from_args(name, dtype, shape, region, approx_order=1,
-                  space='H1', poly_space_base='lagrange'):
+                  space='H1', poly_space_basis='lagrange'):
         """
         Create a Field subclass instance corresponding to a given space.
 
@@ -107,7 +109,7 @@ class Field(Struct):
             The FE approximation order, e.g. 0, 1, 2, '1B' (1 with bubble).
         space : str
             The function space name.
-        poly_space_base : str
+        poly_space_basis : str
             The name of polynomial space base.
 
         Notes
@@ -116,7 +118,7 @@ class Field(Struct):
         """
         conf = Struct(name=name, dtype=dtype, shape=shape, region=region.name,
                       approx_order=approx_order, space=space,
-                      poly_space_base=poly_space_base)
+                      poly_space_basis=poly_space_basis)
         return Field.from_conf(conf, {region.name : region})
 
     @staticmethod
@@ -139,12 +141,19 @@ class Field(Struct):
         table = Field._all
 
         space = conf.get('space', 'H1')
-        poly_space_base = conf.get('poly_space_base', 'lagrange')
+        if 'poly_space_base' in conf.to_dict(): # For backward compatibility.
+            poly_space_basis = conf.get('poly_space_base')
 
-        key = space + '_' + poly_space_base
+        else:
+            poly_space_basis = conf.get('poly_space_basis', 'lagrange')
+
+        key = space + '_' + poly_space_basis
 
         approx_order = parse_approx_order(conf.approx_order)
         ao, force_bubble, discontinuous = approx_order
+        if poly_space_basis == 'constant':
+            discontinuous = False
+
         region = regions[conf.region]
 
         if region.kind == 'cell':
@@ -175,7 +184,7 @@ class Field(Struct):
                         'An abstract Field method called!')
         aux = name.split('_')
         self.space = aux[1]
-        self.poly_space_base = aux[2]
+        self.poly_space_basis = aux[2]
 
     def clear_mappings(self, clear_all=False):
         """
@@ -219,9 +228,9 @@ class Field(Struct):
 
         Returns
         -------
-        geo : CMapping instance
+        geo : PyCMapping instance
             The reference mapping.
-        mapping : VolumeMapping or SurfaceMapping instance
+        mapping : FEMapping or IGMapping instance
             The mapping.
         key : tuple
             The key of the mapping in `mappings` or `mappings0`.
@@ -250,6 +259,84 @@ class Field(Struct):
 
         return out
 
+    def set_dofs(self, fun=0.0, region=None, dpn=None, warn=None):
+        """
+        Set the values of DOFs in a given `region` using a function of space
+        coordinates or value `fun`.
+
+        If `fun` is a function, the l2 projection that is global for all region
+        facets is used to set the DOFs.
+
+        If `dpn > 1`, and `fun` is a function, it has to return the values
+        point-by-point, i.e. all components in the first point, in the second
+        point etc., concatenated to an array that is reshapable to the shape
+        `(n_point, dpn)`.
+
+        Parameters
+        ----------
+        fun : float or array of length dpn or callable
+            The DOF values.
+        region : Region
+            The region containing the DOFs.
+        dpn : int, optional
+            The DOF-per-node count. If not given, the number of field
+            components is used.
+        warn : str, optional
+            The warning message printed when the region selects no DOFs.
+
+        Returns
+        -------
+        nods : array, shape (n_dof,)
+            The field DOFs (or node indices) given by the region.
+        vals : array, shape (n_dof, dpn)
+            The values of the DOFs, node-by-node when raveled in C (row-major)
+            order.
+
+        Notes
+        -----
+        The nodal basis fields (lagrange) reimplement this function to set DOFs
+        directly.
+
+        The hierarchical basis field (lobatto) do not support surface mappings,
+        so also reimplement this function.
+        """
+        if region is None:
+            region = self.region
+
+        if dpn is None:
+            dpn = self.n_components
+
+        aux = self.get_dofs_in_region(region)
+        nods = nm.unique(aux)
+
+        if nm.isscalar(fun):
+            vals = nm.full(nods.shape[0] * dpn, fun, dtype=nm.dtype(type(fun)))
+
+        elif isinstance(fun, nm.ndarray):
+            try:
+                assert_(len(fun) == dpn)
+
+            except (TypeError, ValueError):
+                msg = ('wrong array value shape for setting'
+                       ' DOFs of "%s" field!'
+                       ' (shape %s should be %s)'
+                       % (self.name, fun.shape, (dpn,)))
+                raise ValueError(msg)
+
+            vals = nm.tile(fun, nods.shape[0])
+
+        elif callable(fun):
+            from sfepy.discrete.projections import project_to_facets
+
+            vals = project_to_facets(region, fun, dpn, self)
+
+        else:
+            raise ValueError('unknown function/value type! (%s)' % type(fun))
+
+        vals.shape = (len(nods), -1)
+
+        return nods, vals
+
     def create_eval_mesh(self):
         """
         Create a mesh for evaluating the field. The default implementation
@@ -271,9 +358,9 @@ class Field(Struct):
             The coordinates the source values should be interpolated into.
         source_vals : array, shape ``(n_nod, n_components)``
             The source DOF values corresponding to the field.
-        mode : {'val', 'grad'}, optional
-            The evaluation mode: the field value (default) or the field value
-            gradient.
+        mode : {'val', 'grad', 'div', 'cauchy_strain'}, optional
+            The evaluation mode: the field value (default), the field value
+            gradient, divergence, or cauchy strain.
         strategy : {'general', 'convex'}, optional
             The strategy for finding the elements that contain the
             coordinates. For convex meshes, the 'convex' strategy might be
@@ -308,7 +395,7 @@ class Field(Struct):
         Returns
         -------
         vals : array
-            The interpolated values with shape ``(n_coor, n_components)`` or
+            The interpolated values with shape ``(n_coor, n_components, 1)`` or
             gradients with shape ``(n_coor, n_components, dim)`` according to
             the `mode`. If `ret_status` is False, the values where the status
             is greater than one are set to ``numpy.nan``.
@@ -341,41 +428,45 @@ class Field(Struct):
 
         timer = Timer(start=True)
 
+        n_comp, nc, dim = source_vals.shape[1], coors.shape[0], coors.shape[1]
         # Interpolate to the reference coordinates.
         source_dtype = nm.float64 if source_vals.dtype in complex_types\
             else source_vals.dtype
         if mode == 'val':
-            vals = nm.empty((coors.shape[0], source_vals.shape[1], 1),
-                            dtype=source_dtype)
+            vals = nm.empty((nc, n_comp, 1), dtype=source_dtype)
             cmode = 0
 
-        elif mode == 'grad':
-            vals = nm.empty((coors.shape[0], source_vals.shape[1],
-                             coors.shape[1]),
-                            dtype=source_dtype)
+        elif mode in ['grad', 'div', 'cauchy_strain']:
+            vals = nm.empty((nc, n_comp, dim), dtype=source_dtype)
             cmode = 1
 
         ctx = self.create_basis_context()
+        econn = self.get_econn(('cell', self.region.tdim), self.region)
 
         if source_vals.dtype in complex_types:
             valsi = vals.copy()
             evaluate_in_rc(vals, ref_coors, cells, status,
                            nm.ascontiguousarray(source_vals.real),
-                           self.get_econn('volume', self.region), cmode, ctx)
+                           econn, cmode, ctx)
             evaluate_in_rc(valsi, ref_coors, cells, status,
                            nm.ascontiguousarray(source_vals.imag),
-                           self.get_econn('volume', self.region), cmode, ctx)
+                           econn, cmode, ctx)
             vals = vals + valsi * 1j
         else:
             evaluate_in_rc(vals, ref_coors, cells, status, source_vals,
-                           self.get_econn('volume', self.region), cmode, ctx)
+                           econn, cmode, ctx)
 
         output('interpolation: %f s' % timer.stop(),verbose=verbose)
 
         output('...done',verbose=verbose)
 
-        if mode == 'val':
-            vals.shape = (coors.shape[0], source_vals.shape[1])
+        if mode == 'div':
+            assert_(n_comp == dim)
+            vals = nm.trace(vals, axis1=1, axis2=2).reshape(nc, 1, 1)
+
+        elif mode == 'cauchy_strain':
+            assert_(n_comp == dim)
+            vals = get_cauchy_strain(vals)
 
         if not ret_status:
             ii = nm.where(status > 1)[0]
