@@ -1,7 +1,6 @@
 """
 Classes of equations composed of terms.
 """
-from __future__ import absolute_import
 from copy import copy
 
 import numpy as nm
@@ -10,10 +9,10 @@ import scipy.sparse as sp
 from sfepy.base.base import output, assert_, get_default, iter_dict_of_lists
 from sfepy.base.base import OneTypeList, Container, Struct
 from sfepy.base.timing import Timer
+from sfepy.linalg.utils import chunk_arrays, cycle
 from sfepy.discrete import Materials, Variables, create_adof_conns
-from sfepy.discrete.common.extmods.cmesh import create_mesh_graph
 from sfepy.terms import Terms, Term
-import six
+from sfepy.terms.terms_multilinear import ETermBase
 
 def parse_definition(equation_def):
     """
@@ -47,23 +46,116 @@ def get_expression_arg_names(expression, strip_dots=True):
 
     return set(args)
 
+def create_dof_graph(rdc, cdc, shape, active_only=False):
+    """
+    Create the DOF graph corresponding to a row and column DOF connectivity.
+    """
+    n_c, n_rd = rdc.shape
+    n_c, n_cd = cdc.shape
+
+    rows = nm.broadcast_to(rdc[..., None], (n_c, n_rd, n_cd)).ravel()
+    cols = nm.broadcast_to(cdc[:, None, :], (n_c, n_rd, n_cd)).ravel()
+    if active_only:
+        mask = (rows >= 0) & (cols >= 0)
+        rows = rows[mask]
+        cols = cols[mask]
+
+    vals = nm.broadcast_to(nm.ones(1, dtype=bool), rows.shape)
+    graph = sp.coo_matrix((vals, (rows, cols)), shape=shape)
+
+    return graph
+
+def create_matrix_graph(rdcs, cdcs, irs, ics, rdi, cdi, active_only=True,
+                        chunk_size=200000):
+    """
+    Created the matrix graph using (active) DOF connectivities.
+
+    Parameters
+    ----------
+    rdcs, cdcs : list of arrays
+        Row and column DOF connectivities, corresponding to the variables used
+        in the equations.
+    irs, ics : list of ints
+        Row and column block indices for each row and column connectivity pair.
+    rdi, cdi : DofInfo
+        Row and column DOF info.
+    active_only : bool
+        If True, the matrix graph has reduced size and is created with the
+        reduced (active DOFs only) numbering.
+    chunk_size : int
+        The maximum number of cells added to the graph in one
+        :func:`create_dof_graph()` call.
+
+    Returns
+    -------
+    graph : boolean csr_matrix
+        The matrix graph.
+    """
+    blocks = []
+    roffs = [ii.start for ii in rdi.indx.values()]
+    coffs = [ii.start for ii in cdi.indx.values()]
+    nrs = [ii.stop - ii.start for ii in rdi.indx.values()]
+    ncs = [ii.stop - ii.start for ii in cdi.indx.values()]
+
+    nbr, nbc = max(irs) + 1, max(ics) + 1
+    blocks = [[0] * nbc for ir in range(nbr)]
+    for ii, rdc in enumerate(rdcs):
+        cdc = cdcs[ii]
+
+        ir = irs[ii]
+        ic = ics[ii]
+
+        # Offset back row, column connectivities to start from 0 - sp.bmat()
+        # takes care of the offsets then.
+        srdc = rdc - roffs[ir]
+        scdc = cdc - coffs[ic]
+
+        shape = (nrs[ir], ncs[ic])
+
+        # N - k C >= 0.5 C, k = N // Cmax, compute C.
+        n_cell = srdc.shape[0]
+        cs = int(n_cell / (n_cell // chunk_size + 0.5))
+        for ichunk, (_rdc, _cdc) in enumerate(chunk_arrays((srdc, scdc), cs)):
+            if ichunk == 0:
+                block = create_dof_graph(_rdc, _cdc, shape, active_only)
+
+            else:
+                block += create_dof_graph(_rdc, _cdc, shape, active_only)
+
+        if isinstance(blocks[ir][ic], int):
+            blocks[ir][ic] = block
+
+        else:
+            blocks[ir][ic] += block
+
+    for ir, ic in cycle((nbr, nbc)):
+        if isinstance(blocks[ir][ic], int):
+            blocks[ir][ic] = None
+
+    graph = sp.bmat(blocks, format='csr')
+
+    return graph
+
 class Equations(Container):
 
     @staticmethod
     def from_conf(conf, variables, regions, materials, integrals,
-                  user=None, verbose=True):
+                  user=None, eterm_options=None, allow_derivatives=False,
+                  verbose=True):
 
         objs = OneTypeList(Equation)
 
         conf = copy(conf)
 
         ii = 0
-        for name, desc in six.iteritems(conf):
+        for name, desc in conf.items():
             if verbose:
                 output('equation "%s":' %  name)
                 output(desc)
             eq = Equation.from_desc(name, desc, variables, regions,
-                                    materials, integrals, user=user)
+                                    materials, integrals, user=user,
+                                    eterm_options=eterm_options,
+                                    allow_derivatives=allow_derivatives)
             objs.append(eq)
             ii += 1
 
@@ -199,16 +291,30 @@ class Equations(Container):
                                  msg_if_none='unknown variable! (%s)' % name)
         return var
 
-    def collect_conn_info(self):
+    def collect_conn_info(self, regions_changed=None):
         """
         Collect connectivity information as defined by the equations.
+
+        If `regions_changed` argument is None, new data are collecterd.
+        Otherwise, update already collected data of terms that use the changed
+        regions.
+
+        Parameters
+        ----------
+        regions_changed : str or iterable of str or None
+            Name(s) of changed regions.
         """
-        self.conn_info = {}
+        if regions_changed is None:
+            self.conn_info = {}
+            cond = None
+        else:
+            if isinstance(regions_changed, str):
+                cond = lambda term: term.region.name == regions_changed
+            else:
+                cond = lambda term: term.region.name in regions_changed
 
         for eq in self:
-            eq.collect_conn_info(self.conn_info)
-
-        return self.conn_info
+            eq.collect_conn_info(self.conn_info, condition=cond)
 
     def get_variable_names(self):
         """
@@ -261,7 +367,7 @@ class Equations(Container):
 
     def time_update(self, ts, ebcs=None, epbcs=None, lcbcs=None,
                     functions=None, problem=None, active_only=True,
-                    verbose=True):
+                    regions_changed=None, verbose=True):
         """
         Update the equations for current time step.
 
@@ -288,6 +394,8 @@ class Equations(Container):
             If True, the active DOF connectivities and matrix graph have
             reduced size and are created with the reduced (active DOFs only)
             numbering.
+        regions_changed : str or iterable of str or None
+            Name(s) of changed regions that need new active connectivities.
         verbose : bool
             If False, reduce verbosity.
 
@@ -298,6 +406,9 @@ class Equations(Container):
             boundary conditions differs from the set of the previous
             time step.
         """
+        if regions_changed:
+            self.collect_conn_info(regions_changed)
+
         self.variables.time_update(ts, functions, verbose=verbose)
 
         active_bcs = self.variables.equation_mapping(ebcs, epbcs, ts, functions,
@@ -311,6 +422,12 @@ class Equations(Container):
                                      active_only=active_only)
             self.variables.set_adof_conns(adcs)
 
+        elif regions_changed:
+            adcs = create_adof_conns(self.conn_info, self.variables.adi.indx,
+                                     active_only=active_only,
+                                     regions_changed=regions_changed)
+            self.variables.update_adof_conns(adcs)
+
         self.variables.setup_lcbc_operators(lcbcs, ts, functions)
 
         for eq in self:
@@ -342,7 +459,9 @@ class Equations(Container):
     def setup_initial_conditions(self, ics, functions=None):
         self.variables.setup_initial_conditions(ics, functions)
 
-    def get_graph_conns(self, any_dof_conn=False, rdcs=None, cdcs=None,
+    def get_graph_conns(self, any_dof_conn=False,
+                        rdcs=None, cdcs=None,
+                        irs=None, ics=None,
                         active_only=True):
         """
         Get DOF connectivities needed for creating tangent matrix graph.
@@ -350,38 +469,53 @@ class Equations(Container):
         Parameters
         ----------
         any_dof_conn : bool
-            By default, only volume DOF connectivities are used, with
-            the exception of trace surface DOF connectivities. If True,
+            By default, only cell DOF connectivities are used, with
+            the exception of trace facet DOF connectivities. If True,
             any kind of DOF connectivities is allowed.
-        rdcs, cdcs : arrays, optional
+        rdcs, cdcs : list of arrays, optional
             Additional row and column DOF connectivities, corresponding
             to the variables used in the equations.
+        irs, ics: list of ints, optional
+            Additional row and column block indices for each row and column
+            connectivity pair.
         active_only : bool
             If True, the active DOF connectivities have reduced size and are
             created with the reduced (active DOFs only) numbering.
 
         Returns
         -------
-        rdcs, cdcs : arrays
+        rdcs, cdcs : list of arrays
             The row and column DOF connectivities defining the matrix
             graph blocks.
+        irs, ics : list of ints
+            Row and column block indices for each row and column connectivity
+            pair.
         """
         if rdcs is None:
             rdcs = []
             cdcs = []
+            irs = []
+            ics = []
 
         elif cdcs is None:
             cdcs = copy(rdcs)
+            ics = copy(irs)
 
         else:
             assert_(len(rdcs) == len(cdcs))
             if rdcs is cdcs: # Make sure the lists are not the same object.
                 rdcs = copy(rdcs)
 
-        adcs = self.variables.adof_conns
+            assert_(len(irs) == len(ics))
+            if irs is ics:
+                irs = copy(ics)
 
-        # Only volume dof connectivities are used, with the exception of trace
-        # surface dof connectivities.
+        adcs = self.variables.adof_conns
+        rdi = self.variables.avdi
+        cdi = self.variables.adi
+
+        # Only cell dof connectivities are used, with the exception of trace
+        # facet dof connectivities.
         shared = set()
         for key, ii, info in iter_dict_of_lists(self.conn_info,
                                                 return_keys=True):
@@ -389,23 +523,19 @@ class Equations(Container):
             if (rvar is None) or (cvar is None):
                 continue
 
-            is_surface = rvar.is_surface or cvar.is_surface
-
-            dct = info.dc_type.type
-            if not (dct in ('volume', 'scalar', 'custom') or is_surface
-                    or info.is_trace or any_dof_conn):
-                continue
-
             rreg_name = info.get_region_name(can_trace=False)
             creg_name = info.get_region_name()
+            mreg_name = None if creg_name == rreg_name else rreg_name
 
             rname = rvar.get_primary_name()
-            rkey = (rname, rreg_name, dct, False)
-            ckey = (cvar.name, creg_name, dct, info.is_trace)
+            cname = cvar.get_primary_name()
+            rkey = (rname, rreg_name, info.dof_conn_types[rname], None)
+            ckey = (cvar.name, creg_name, info.dof_conn_types[cname],
+                    mreg_name)
 
             dc_key = (rkey, ckey)
 
-            if not dc_key in shared:
+            if dc_key not in shared:
                 rdc = adcs[rkey]
                 cdc = adcs[ckey]
                 if not active_only:
@@ -420,12 +550,20 @@ class Equations(Container):
                 rdcs.append(rdc)
                 cdcs.append(cdc)
 
+                irs.append(rdi.indx[rvar.get_primary_name()].start)
+                ics.append(cdi.indx[cvar.get_primary_name()].start)
+
                 shared.add(dc_key)
 
-        return rdcs, cdcs
+        # Replace offsets by their orders.
+        _, irs = nm.unique(irs, return_inverse=True)
+        _, ics = nm.unique(ics, return_inverse=True)
+
+        return rdcs, cdcs, irs, ics
 
     def create_matrix_graph(self, any_dof_conn=False, rdcs=None, cdcs=None,
-                            shape=None, active_only=True, verbose=True):
+                            shape=None, active_only=True, chunk_size=200000,
+                            verbose=True):
         """
         Create tangent matrix graph, i.e. preallocate and initialize the
         sparse storage needed for the tangent matrix. Order of DOF
@@ -434,9 +572,9 @@ class Equations(Container):
         Parameters
         ----------
         any_dof_conn : bool
-            By default, only volume DOF connectivities are used, with
-            the exception of trace surface DOF connectivities. If True,
-            any kind of DOF connectivities is allowed.
+            By default, only cell region DOF connectivities are used, with
+            the exception of trace facet DOF connectivities. If True,
+            any DOF connectivities are used.
         rdcs, cdcs : arrays, optional
             Additional row and column DOF connectivities, corresponding
             to the variables used in the equations.
@@ -447,6 +585,9 @@ class Equations(Container):
         active_only : bool
             If True, the matrix graph has reduced size and is created with the
             reduced (active DOFs only) numbering.
+        chunk_size : int
+            The maximum number of cells added to the graph in one
+            :func:`create_dof_graph()` call.
         verbose : bool
             If False, reduce verbosity.
 
@@ -468,9 +609,9 @@ class Equations(Container):
             output('no matrix (zero size)!')
             return None
 
-        rdcs, cdcs = self.get_graph_conns(any_dof_conn=any_dof_conn,
-                                          rdcs=rdcs, cdcs=cdcs,
-                                          active_only=active_only)
+        rdcs, cdcs, irs, ics = self.get_graph_conns(any_dof_conn=any_dof_conn,
+                                                    rdcs=rdcs, cdcs=cdcs,
+                                                    active_only=active_only)
 
         if not len(rdcs):
             output('no matrix (empty dof connectivities)!')
@@ -479,12 +620,16 @@ class Equations(Container):
         output('assembling matrix graph...', verbose=verbose)
         timer = Timer(start=True)
 
-        nnz, prow, icol = create_mesh_graph(shape[0], shape[1],
-                                            len(rdcs), rdcs, cdcs)
+        rdi = self.variables.avdi
+        cdi = self.variables.adi
+        gr = create_matrix_graph(rdcs, cdcs, irs, ics, rdi, cdi,
+                                 active_only=active_only,
+                                 chunk_size=chunk_size)
+        nnz, prow, icol = gr.nnz, gr.indptr, gr.indices
 
         output('...done in %.2f s' % timer.stop(), verbose=verbose)
         output('matrix structural nonzeros: %d (%.2e%% fill)' \
-               % (nnz, float(nnz) / size), verbose=verbose)
+               % (nnz, 100.0 * float(nnz) / size), verbose=verbose)
 
         data = nm.zeros((nnz,), dtype=self.variables.dtype)
         matrix = sp.csr_matrix((data, icol, prow), shape)
@@ -503,15 +648,15 @@ class Equations(Container):
 
     ##
     # Interface to self.variables.
-    def create_state_vector(self):
-        return self.variables.create_state_vector()
+    def create_vec(self):
+        return self.variables.create_vec()
 
-    def create_stripped_state_vector(self):
-        return self.variables.create_stripped_state_vector()
+    def create_reduced_vec(self):
+        return self.variables.create_reduced_vec()
 
-    def strip_state_vector(self, vec, follow_epbc=False):
+    def reduce_vec(self, vec, follow_epbc=False):
         """
-        Strip a full vector by removing EBC dofs.
+        Get the reduced DOF vector, with EBC and PBC DOFs removed.
 
         Notes
         -----
@@ -520,7 +665,7 @@ class Equations(Container):
         assembling. For vectors with state (unknown) variables it should be set
         to False, for assembled vectors it should be set to True.
         """
-        return self.variables.strip_state_vector(vec, follow_epbc=follow_epbc)
+        return self.variables.reduce_vec(vec, follow_epbc=follow_epbc)
 
     def make_full_vec(self, svec, force_value=None):
         """
@@ -529,44 +674,13 @@ class Equations(Container):
         """
         return self.variables.make_full_vec(svec, force_value)
 
-    def set_variables_from_state(self, vec, step=0):
-        """
-        Set data (vectors of DOF values) of variables.
-
-        Parameters
-        ----------
-        data : array
-            The state vector.
-        step : int
-            The time history step, 0 (default) = current.
-        """
-        self.variables.set_data(vec, step=step)
-
-    def get_state_parts(self, vec=None):
-        """
-        Return parts of a state vector corresponding to individual state
-        variables.
-
-        Parameters
-        ----------
-        vec : array, optional
-            The state vector. If not given, then the data stored in the
-            variables are returned instead.
-
-        Returns
-        -------
-        out : dict
-            The dictionary of the state parts.
-        """
-        return self.variables.get_state_parts(vec)
-
     def set_data(self, data, step=0, ignore_unknown=False):
         """
         Set data (vectors of DOF values) of variables.
 
         Parameters
         ----------
-        data : array
+        data : dict
             The dictionary of {variable_name : data vector}.
         step : int, optional
             The time history step, 0 (default) = current.
@@ -576,39 +690,61 @@ class Equations(Container):
         self.variables.set_data(data, step=step,
                                 ignore_unknown=ignore_unknown)
 
-    def apply_ebc(self, vec, force_values=None):
-        """
-        Apply essential (Dirichlet) boundary conditions to a state vector.
-        """
-        self.variables.apply_ebc(vec, force_values=force_values)
+    def init_state(self, vec=None):
+        self.variables.init_state(vec=vec)
 
-    def apply_ic(self, vec, force_values=None):
+    def apply_ebc(self, vec=None, force_values=None):
         """
-        Apply initial conditions to a state vector.
+        Apply essential (Dirichlet) boundary conditions to equations' variables,
+        or a given vector.
         """
-        self.variables.apply_ic(vec, force_values=force_values)
+        self.variables.apply_ebc(vec=vec, force_values=force_values)
 
-    def state_to_output(self, vec, fill_value=None, var_info=None,
-                        extend=True):
-        return self.variables.state_to_output(vec,
-                                              fill_value=fill_value,
-                                              var_info=var_info,
-                                              extend=extend)
+    def apply_ic(self, vec=None, force_values=None):
+        """
+        Apply initial conditions to equations' variables, or a given vector.
+        """
+        self.variables.apply_ic(vec=vec, force_values=force_values)
+
+    def set_state(self, vec, reduced=False, force=False, preserve_caches=False):
+        self.variables.set_state(vec, reduced=reduced, force=force,
+                                 preserve_caches=preserve_caches)
 
     def get_lcbc_operator(self):
         return self.variables.get_lcbc_operator()
 
     def evaluate(self, names=None, mode='eval', dw_mode='vector',
-                 term_mode=None, asm_obj=None):
+                 term_mode=None, diff_vars=None, asm_obj=None,
+                 select_term=None,
+                 assemble=None):
         """
         Evaluate the equations.
 
         Parameters
         ----------
-        mode : one of 'eval', 'el_avg', 'qp', 'weak'
-            The evaluation mode.
         names : str or sequence of str, optional
             Evaluate only equations of the given name(s).
+        mode : one of 'eval', 'el_avg', 'qp', 'weak'
+            The evaluation mode.
+        dw_mode : one of 'vector', 'matrix', 'sensitivity'
+            The particular evaluation mode if `mode` is ``'weak'``.
+        term_mode : str
+            The term evaluation mode, used mostly if `mode` is ``'eval'`` in
+            some terms.
+        diff_vars : list of str
+            The names of parameters with respect to the equations are
+            differentiated if `dw_mode` is ``'sensitivity'``.
+        asm_obj : ndarray or spmatrix
+            The object for storing the evaluation result in the ``'weak'`` mode.
+        select_term : function(term)
+            Optional boolean function returning True for terms that should be
+            evaluated.
+        assemble : function(*args, **kwargs)
+            Optional alternative FE assembling function with arguments:
+            (equation, it, term, asm_obj, vals, iels, mode=, diff_var=), where
+            `term` is the evaluated term, `it` is the term's order in the
+            `equation` and (asm_obj, vals, iels, mode=, diff_var=) are the
+            Term.assemble_to() arguments.
 
         Returns
         -------
@@ -632,7 +768,10 @@ class Equations(Container):
             extras = []
             for eq in eqs:
                 out = eq.evaluate(mode=mode, dw_mode=dw_mode,
-                                  term_mode=term_mode, asm_obj=asm_obj)
+                                  term_mode=term_mode, diff_vars=diff_vars,
+                                  asm_obj=asm_obj,
+                                  select_term=select_term,
+                                  assemble=assemble)
                 if isinstance(out, tuple): extras.extend(out[1])
 
             out = asm_obj
@@ -643,7 +782,9 @@ class Equations(Container):
             out = {}
             for eq in eqs:
                 eout = eq.evaluate(mode=mode, dw_mode=dw_mode,
-                                   term_mode=term_mode)
+                                   term_mode=term_mode,
+                                   select_term=select_term,
+                                   assemble=assemble)
                 out[eq.name] = eout
 
             if single:
@@ -651,7 +792,9 @@ class Equations(Container):
 
         return out
 
-    def eval_residuals(self, state, by_blocks=False, names=None):
+    def eval_residuals(self, state, by_blocks=False, names=None,
+                       by_terms=False, select_term=None,
+                       assemble=None):
         """
         Evaluate (assemble) residual vectors.
 
@@ -668,6 +811,18 @@ class Equations(Container):
         names : list of str, optional
             Optionally, select only blocks with the given `names`, if
             `by_blocks` is True.
+        by_terms : bool
+            If True, return a dict of the individual term residuals indexed by
+            (<equation name>, <term index>).
+        select_term : function(term)
+            Optional boolean function returning True for terms that should be
+            evaluated.
+        assemble : function(*args, **kwargs)
+            Optional alternative FE assembling function with arguments:
+            (equation, it, term, asm_obj, vals, iels, mode=, diff_var=), where
+            `term` is the evaluated term, `it` is the term's order in the
+            `equation` and (asm_obj, vals, iels, mode=, diff_var=) are the
+            Term.assemble_to() arguments.
 
         Returns
         -------
@@ -676,7 +831,7 @@ class Equations(Container):
             dictionary is returned instead, with keys given by
             `block_name` part of the individual equation names.
         """
-        self.set_variables_from_state(state)
+        self.set_state(state, force=True)
 
         if by_blocks:
             names = get_default(names, self.names)
@@ -689,22 +844,48 @@ class Equations(Container):
                 key, rname, cname = [aux.strip()
                                      for aux in name.split(',')]
 
-                ir = get_indx(rname, stripped=True, allow_dual=True)
+                ir = get_indx(rname, reduced=True, allow_dual=True)
 
-                residual = self.create_stripped_state_vector()
-                eq.evaluate(mode='weak', dw_mode='vector', asm_obj=residual)
+                residual = self.create_reduced_vec()
+                eq.evaluate(mode='weak', dw_mode='vector', asm_obj=residual,
+                            select_term=select_term, assemble=assemble)
 
                 out[key] = residual[ir]
 
-        else:
-            out = self.create_stripped_state_vector()
+        elif by_terms:
+            out = {}
 
-            self.evaluate(mode='weak', dw_mode='vector', asm_obj=out)
+            get_indx = self.variables.get_indx
+            for eq in self:
+                for it, term in enumerate(eq.terms):
+                    if select_term:
+                        _select_term = (lambda x: select_term(x)
+                                        and (x.name == term.name))
+
+                    else:
+                        _select_term = lambda x: x.name == term.name
+
+                    ir = get_indx(term.get_virtual_name(),
+                                  reduced=True, allow_dual=True)
+
+                    residual = self.create_reduced_vec()
+                    eq.evaluate(mode='weak', dw_mode='vector', asm_obj=residual,
+                                select_term=_select_term, assemble=assemble)
+
+                    out[(eq.name, it)] = residual[ir]
+
+        else:
+            out = self.create_reduced_vec()
+
+            self.evaluate(mode='weak', dw_mode='vector', asm_obj=out,
+                          select_term=select_term, assemble=assemble)
 
         return out
 
     def eval_tangent_matrices(self, state, tangent_matrix,
-                              by_blocks=False, names=None):
+                              by_blocks=False, names=None,
+                              by_terms=False, select_term=None,
+                              assemble=None):
         """
         Evaluate (assemble) tangent matrices.
 
@@ -723,6 +904,18 @@ class Equations(Container):
         names : list of str, optional
             Optionally, select only blocks with the given `names`, if
             `by_blocks` is True.
+        by_terms : bool
+            If True, return a dict of the individual term matrices indexed by
+            (<equation name>, <term index>).
+        select_term : function(term)
+            Optional boolean function returning True for terms that should be
+            evaluated.
+        assemble : function(*args, **kwargs)
+            Optional alternative FE assembling function with arguments:
+            (equation, it, term, asm_obj, vals, iels, mode=, diff_var=), where
+            `term` is the evaluated term, `it` is the term's order in the
+            `equation` and (asm_obj, vals, iels, mode=, diff_var=) are the
+            Term.assemble_to() arguments.
 
         Returns
         -------
@@ -731,7 +924,7 @@ class Equations(Container):
             is returned instead, with keys given by `block_name` part
             of the individual equation names.
         """
-        self.set_variables_from_state(state)
+        self.set_state(state, force=True)
 
         if by_blocks:
             names = get_default(names, self.names)
@@ -744,20 +937,50 @@ class Equations(Container):
                 key, rname, cname = [aux.strip()
                                      for aux in eq.name.split(',')]
 
-                ir = get_indx(rname, stripped=True, allow_dual=True)
-                ic = get_indx(cname, stripped=True, allow_dual=True)
+                ir = get_indx(rname, reduced=True, allow_dual=True)
+                ic = get_indx(cname, reduced=True, allow_dual=True)
 
                 tangent_matrix.data[:] = 0.0
                 aux = eq.evaluate(mode='weak', dw_mode='matrix',
-                                  asm_obj=tangent_matrix)
+                                  asm_obj=tangent_matrix,
+                                  select_term=select_term,
+                                  assemble=assemble)
 
                 out[key] = aux[ir, ic]
 
+        elif by_terms:
+            out = {}
+
+            get_indx = self.variables.get_indx
+            for eq in self:
+                for it, term in enumerate(eq.terms):
+                    if select_term:
+                        _select_term = (lambda x: select_term(x)
+                                        and (x.name == term.name))
+
+                    else:
+                        _select_term = lambda x: x.name == term.name
+
+                    ir = get_indx(term.get_virtual_name(),
+                                  reduced=True, allow_dual=True)
+
+                    tangent_matrix.data[:] = 0.0
+                    eq.evaluate(mode='weak', dw_mode='matrix',
+                                asm_obj=tangent_matrix,
+                                select_term=_select_term,
+                                assemble=assemble)
+
+                    # All columns (possibly related to several variables) are
+                    # included.
+                    out[(eq.name, it)] = tangent_matrix[ir]
+
         else:
             tangent_matrix.data[:] = 0.0
 
             out = self.evaluate(mode='weak', dw_mode='matrix',
-                                asm_obj=tangent_matrix)
+                                asm_obj=tangent_matrix,
+                                select_term=select_term,
+                                assemble=assemble)
 
         return out
 
@@ -765,18 +988,24 @@ class Equation(Struct):
 
     @staticmethod
     def from_desc(name, desc, variables, regions, materials, integrals,
-                  user=None):
+                  user=None, eterm_options=None, allow_derivatives=False):
         term_descs = parse_definition(desc)
         terms = Terms.from_desc(term_descs, regions, integrals)
 
-        terms.setup()
+        terms.setup(allow_derivatives=allow_derivatives)
         terms.assign_args(variables, materials, user)
 
-        obj = Equation(name, terms)
+        if eterm_options is not None:
+            for term in terms:
+                if isinstance(term, ETermBase):
+                    term.set_verbosity(eterm_options.get('verbosity', 0))
+                    term.set_backend(**eterm_options.get('backend_args', {}))
+
+        obj = Equation(name, terms, setup=False)
 
         return obj
 
-    def __init__(self, name, terms):
+    def __init__(self, name, terms, setup=True):
         Struct.__init__(self, name=name)
 
         if isinstance(terms, Term): # A single term.
@@ -784,7 +1013,8 @@ class Equation(Struct):
 
         self.terms = terms
 
-        self.terms.setup()
+        if setup:
+            self.terms.setup()
 
     def collect_materials(self):
         """
@@ -817,24 +1047,63 @@ class Equation(Struct):
 
         return variables
 
-    def collect_conn_info(self, conn_info):
+    def collect_conn_info(self, conn_info, condition=None):
 
         for term in self.terms:
+            if condition and not condition(term):
+                continue
+
             key = (self.name,) + term.get_conn_key()
 
             conn_info[key] = term.get_conn_info()
 
     def evaluate(self, mode='eval', dw_mode='vector', term_mode=None,
-                 asm_obj=None):
+                 diff_vars=None, asm_obj=None, select_term=None,
+                 assemble=None):
         """
+        Evaluate the equation.
+
         Parameters
         ----------
         mode : one of 'eval', 'el_eval', 'el_avg', 'qp', 'weak'
             The evaluation mode.
+        dw_mode : one of 'vector', 'matrix', 'sensitivity'
+            The particular evaluation mode if `mode` is ``'weak'``.
+        term_mode : str
+            The term evaluation mode, used mostly if `mode` is ``'eval'`` in
+            some terms.
+        diff_vars : list of str
+            The names of parameters with respect to the equation is
+            differentiated if `dw_mode` is ``'sensitivity'``.
+        asm_obj : ndarray or spmatrix
+            The object for storing the evaluation result in the ``'weak'`` mode.
+        select_term : function(term)
+            Optional boolean function returning True for terms that should be
+            evaluated.
+        assemble : function(*args, **kwargs)
+            Optional alternative FE assembling function with arguments:
+            (equation, it, term, asm_obj, vals, iels, mode=, diff_var=), where
+            `term` is the evaluated term, `it` is the term's order in the
+            `equation` and (asm_obj, vals, iels, mode=, diff_var=) are the
+            Term.assemble_to() arguments.
+
+        Returns
+        -------
+        out : result
+            The evaluation result. In 'weak' mode it is the
+            `asm_obj`.
         """
+        terms = self.terms
+        if select_term is not None:
+            terms = [term for term in self.terms if select_term(term)]
+
+        if assemble is None:
+            assemble = (lambda name, it, term, *args, **kwargs:
+                        term.assemble_to(*args, **kwargs))
+
         if mode in ('eval', 'el_eval', 'el_avg', 'qp'):
             val = 0.0
-            for term in self.terms:
+            for term in terms:
                 aux, status = term.evaluate(mode=mode,
                                             term_mode=term_mode,
                                             standalone=False,
@@ -847,19 +1116,19 @@ class Equation(Struct):
 
             if dw_mode == 'vector':
 
-                for term in self.terms:
+                for it, term in enumerate(terms):
                     val, iels, status = term.evaluate(mode=mode,
                                                       term_mode=term_mode,
                                                       standalone=False,
                                                       ret_status=True)
-                    term.assemble_to(asm_obj, val, iels, mode=dw_mode)
+                    assemble(self, it, term, asm_obj, val, iels, mode=dw_mode)
 
                 out = asm_obj
 
             elif dw_mode == 'matrix':
 
                 extras = []
-                for term in self.terms:
+                for it, term in enumerate(terms):
                     svars = term.get_state_variables(unknown_only=True)
 
                     for svar in svars:
@@ -868,12 +1137,30 @@ class Equation(Struct):
                                                           diff_var=svar.name,
                                                           standalone=False,
                                                           ret_status=True)
-                        extra = term.assemble_to(asm_obj, val, iels,
-                                                 mode=dw_mode, diff_var=svar)
+                        extra = assemble(self, it, term, asm_obj, val, iels,
+                                         mode=dw_mode, diff_var=svar)
                         if extra is not None: extras.append(extra)
 
                 out = (asm_obj, extras) if len(extras) else asm_obj
 
+            elif dw_mode == 'sensitivity':
+                # Differentiation w.r.t. material parameters.
+                if diff_vars is None: diff_vars = ()
+
+                for ic, diff_var in enumerate(diff_vars):
+                    for it, term in enumerate(terms):
+                        if not (term.diff_info and
+                                (diff_var in term.get_material_names(part=1))):
+                            continue
+                        val, iels, status = term.evaluate(mode=mode,
+                                                          term_mode=term_mode,
+                                                          diff_var=diff_var,
+                                                          standalone=False,
+                                                          ret_status=True)
+                        assemble(self, it, term, asm_obj[:, ic], val, iels)
+
+                out = asm_obj
+
             else:
                 raise ValueError('unknown assembling mode! (%s)' % dw_mode)
 
