@@ -10,7 +10,7 @@ from sfepy.discrete import Domain, PolySpace
 from sfepy.discrete.fem.refine import refine_2_3, refine_2_4, refine_3_4, \
     refine_3_8, refine_1_2
 from sfepy.discrete.fem.fe_surface import FESurface
-import six
+
 
 class FEDomain(Domain):
     """
@@ -30,10 +30,6 @@ class FEDomain(Domain):
         """
         Domain.__init__(self, name, mesh=mesh, verbose=verbose, **kwargs)
 
-        if len(mesh.descs) > 1:
-            msg = 'meshes with several cell kinds are not supported!'
-            raise NotImplementedError(msg)
-
         self.geom_els = geom_els = {}
         for ig, desc in enumerate(mesh.descs):
             gel = GeometryElement(desc)
@@ -44,27 +40,35 @@ class FEDomain(Domain):
 
             geom_els[desc] = gel
 
-        for gel in six.itervalues(geom_els):
+        for gel in geom_els.values():
             key = gel.get_interpolation_name()
 
             gel.poly_space = PolySpace.any_from_args(key, gel, 1)
             gel = gel.surface_facet
-            if gel is not None:
+            if isinstance(gel, dict):
+                for v in gel.values():
+                    key = v.get_interpolation_name()
+                    v.poly_space = PolySpace.any_from_args(key, v, 1)
+            elif gel is not None:
                 key = gel.get_interpolation_name()
                 gel.poly_space = PolySpace.any_from_args(key, gel, 1)
 
         self.vertex_set_bcs = self.mesh.nodal_bcs
-
-        self.cmesh = self.mesh.cmesh
+        self.cmesh_tdim = self.mesh.cmesh_tdim
 
         # Must be before creating derived connectivities.
         self.fix_element_orientation()
 
         from sfepy.discrete.fem.geometry_element import create_geometry_elements
         gels = create_geometry_elements()
-        self.cmesh.set_local_entities(gels)
-        self.cmesh.setup_entities()
+        max_tdim = 0
+        for cmesh in self.mesh.cmesh_tdim:
+            if cmesh is not None:
+                cmesh.set_local_entities(gels)
+                cmesh.setup_entities()
+                max_tdim = max(max_tdim, cmesh.tdim)
 
+        self.cmesh = self.cmesh_tdim[max_tdim]
         n_nod, dim = self.mesh.coors.shape
         self.shape = Struct(n_nod=n_nod, dim=dim, tdim=self.cmesh.tdim,
                             n_el=self.cmesh.n_el,
@@ -105,36 +109,40 @@ class FEDomain(Domain):
         bbox = self.get_mesh_bounding_box()
         return (bbox[1,:] - bbox[0,:]).max()
 
-    def fix_element_orientation(self):
+    def fix_element_orientation(self, geom_els=None, force_check=False):
         """
         Ensure element vertices ordering giving positive cell volumes.
         """
         from sfepy.discrete.common.extmods.cmesh import orient_elements
 
-        if self.cmesh.tdim != self.cmesh.dim:
-            output('warning: mesh with topological dimension %d lower than'
-                   ' space dimension %d' % (self.cmesh.tdim, self.cmesh.dim))
-            output('- element orientation not checked!')
-            return
+        if geom_els is None:
+            geom_els = self.geom_els
 
-        cmesh = self.cmesh
-        for key, gel in six.iteritems(self.geom_els):
+        for key, gel in geom_els.items():
             ori = gel.orientation
+            cmesh = self.cmesh_tdim[gel.dim]
+            if (cmesh.tdim != cmesh.dim) and (not force_check):
+                output('warning: mesh with topological dimension %d lower than'
+                       ' space dimension %d' % (cmesh.tdim, cmesh.dim))
+                output('- element orientation not checked!')
+                return
 
             cells = nm.where(cmesh.cell_types == cmesh.key_to_index[gel.name])
             cells = cells[0].astype(nm.uint32)
 
             itry = 0
             while itry < 2:
-                flag = -nm.ones(self.cmesh.n_el, dtype=nm.int32)
+                flag = -nm.ones(cmesh.n_el, dtype=nm.int32)
 
                 # Changes orientation if it is wrong according to swap*!
                 # Changes are indicated by positive flag.
-                orient_elements(flag, self.cmesh, cells, gel.dim,
+                orient_elements(flag, cmesh, cells, gel.dim,
                                 ori.roots, ori.vecs,
                                 ori.swap_from, ori.swap_to)
 
-                if nm.alltrue(flag == 0):
+                flag = flag[cells]
+
+                if nm.all(flag == 0):
                     if itry > 0: output('...corrected')
                     itry = -1
                     break
@@ -148,37 +156,61 @@ class FEDomain(Domain):
             elif flag[0] == -1:
                 output('warning: element orienation not checked')
 
-    def get_conn(self, ret_gel=False):
+    def get_conn(self, ret_gel=False, tdim=None, cells=None):
         """
         Get the cell-vertex connectivity and, if `ret_gel` is True, also the
-        corresponding reference geometry element.
+        corresponding reference geometry element. If `tdim` is not None get
+        the connectivity of the cells with topological dimension `tdim`.
         """
-        conn = self.cmesh.get_conn(self.cmesh.tdim, 0).indices
-        conn = conn.reshape((self.cmesh.n_el, -1)).astype(nm.int32)
+        cmesh = self.cmesh if tdim is None else self.cmesh_tdim[tdim]
+        if cells is None:
+            conn = cmesh.get_conn(cmesh.tdim, 0).indices
+            conn = conn.reshape((cmesh.n_el, -1)).astype(nm.int32)
+        else:
+            conn = cmesh.get_conn(cmesh.tdim, 0)
+            aux = conn.offsets[cells]
+            nv = set(conn.offsets[cells + 1] - aux)
+            if len(nv) != 1:
+                raise ValueError('different cell types in a region!')
 
+            nc = cells.shape[0]
+            nv = nv.pop()
+            idxs = (nm.repeat(conn.offsets[cells], nv) +
+                    nm.tile(nm.arange(nv), nc))
+            conn = conn.indices[idxs].reshape((nc, nv))
+
+        name = f'{cmesh.tdim}_{conn.shape[1]}'
         if ret_gel:
-            gel = list(self.geom_els.values())[0]
+            gel = None
+            for gv in self.geom_els.values():
+                if gv.name == name:
+                    gel = gv
+                    break
 
             return conn, gel
 
         else:
             return conn
 
-    def get_element_diameters(self, cells, vg, mode, square=True):
-        diameters = nm.empty((len(cells), 1, 1, 1), dtype=nm.float64)
-        if vg is None:
-            diameters.fill(1.0)
-        else:
-            conn, gel = self.get_conn(ret_gel=True)
-            vg.get_element_diameters(diameters, gel.edges,
-                                     self.get_mesh_coors().copy(), conn,
-                                     cells.astype(nm.int32), mode)
-        if square:
-            out = diameters.squeeze()
-        else:
-            out = nm.sqrt(diameters.squeeze())
+    def get_element_diameters(self, cells, volume, mode, square=True):
+        conn, gel = self.get_conn(ret_gel=True)
+        coors = self.get_mesh_coors()
+        lconn = conn[cells]
+        ed = gel.edges
 
-        return out
+        if mode == 0 or mode == 2:
+            vv = coors[lconn[:, ed[:, 1]]] - coors[lconn[:, ed[:, 0]]]
+            v0 = nm.max(nm.sum(vv**2, axis=2), axis=1)
+            out = v0
+
+        if mode == 1 or mode == 2:
+            v1 = nm.power(0.16 * volume, 1.0 / gel.dim).squeeze()
+            out = v1
+
+        if mode == 2:
+            out = nm.max(nm.stack([v0, v1]), axis=0)
+
+        return out if square else nm.sqrt(out)
 
     def clear_surface_groups(self):
         """
@@ -194,15 +226,12 @@ class FEDomain(Domain):
         Notes
         -----
         Surface groups define surface facet connectivity that is needed
-        for :class:`sfepy.discrete.fem.mappings.SurfaceMapping`.
+        for :class:`sfepy.discrete.fem.mappings.FEMapping`.
         """
         groups = self.surface_groups
         if region.name not in groups:
-            conn, gel = self.get_conn(ret_gel=True)
-            gel_faces = gel.get_surface_entities()
-
             name = 'surface_group_%s' % (region.name)
-            surface_group = FESurface(name, region, gel_faces, conn)
+            surface_group = FESurface.from_region(name, region)
 
             groups[region.name] = surface_group
 
