@@ -3,16 +3,19 @@ import numpy as nm
 from sfepy.base.base import assert_
 from sfepy.discrete.fem.utils import prepare_remap, prepare_translate
 from sfepy.discrete.common.dof_info import expand_nodes_to_dofs
-from sfepy.discrete.fem.fields_base import VolumeField, H1Mixin
+from sfepy.discrete.fem.fields_base import FEField, H1Mixin
 
-class H1HierarchicVolumeField(H1Mixin, VolumeField):
+class H1HierarchicVolumeField(H1Mixin, FEField):
+    """
+    Hierarchical basis approximation with Lobatto polynomials.
+    """
     family_name = 'volume_H1_lobatto'
 
     def _init_econn(self):
         """
         Initialize the extended DOF connectivity and facet orientation array.
         """
-        VolumeField._init_econn(self)
+        FEField._init_econn(self)
 
         self.ori = nm.zeros_like(self.econn)
 
@@ -49,7 +52,7 @@ class H1HierarchicVolumeField(H1Mixin, VolumeField):
         facet_desc = nm.array(facet_desc)
         n_dof_per_facet = facet_desc.shape[1]
 
-        cmesh = self.domain.cmesh
+        cmesh = self.cmesh
 
         facets = self.region.entities[dim]
         ii = nm.arange(facets.shape[0], dtype=nm.int32)
@@ -58,7 +61,7 @@ class H1HierarchicVolumeField(H1Mixin, VolumeField):
         # Prepare global facet id remapping to field-local numbering.
         remap = prepare_remap(facets, cmesh.num[dim])
 
-        cconn = self.region.domain.cmesh.get_conn(self.region.tdim, dim)
+        cconn = cmesh.get_conn(self.region.tdim, dim)
         offs = cconn.offsets
 
         n_f = self.gel.edges.shape[0] if dim == 1 else self.gel.faces.shape[0]
@@ -170,7 +173,7 @@ class H1HierarchicVolumeField(H1Mixin, VolumeField):
         n_dof_per_cell = self.node_desc.bubble.shape[0]
 
         ii = self.region.get_cells()
-        remap = prepare_remap(ii, self.domain.cmesh.n_el)
+        remap = prepare_remap(ii, self.cmesh.n_el)
 
         n_cell = ii.shape[0]
         n_dof = n_dof_per_cell * n_cell
@@ -184,8 +187,8 @@ class H1HierarchicVolumeField(H1Mixin, VolumeField):
 
     def set_dofs(self, fun=0.0, region=None, dpn=None, warn=None):
         """
-        Set the values of given DOFs using a function of space coordinates or
-        value `fun`.
+        Set the values of DOFs in a given `region` using a function of space
+        coordinates or value `fun`.
         """
         if region is None:
             region = self.region
@@ -202,15 +205,19 @@ class H1HierarchicVolumeField(H1Mixin, VolumeField):
             vals = nm.zeros(n_dof, dtype=nm.dtype(type(fun)))
             vals[:gnods[0].shape[0] * dpn] = fun
 
-
         elif callable(fun):
-            vv = nm.asarray(fun(self.get_coor(gnods[0])))
+            coors = self.get_coor(gnods[0])
+            vv = nm.asarray(fun(coors))
+            if (vv.ndim > 1) and (vv.shape != (len(coors), dpn)):
+                raise ValueError('The projected function return value should be'
+                                 ' (n_point, dpn) == %s, instead of %s!'
+                                 % ((len(coors), dpn), vv.shape))
 
             vals = nm.zeros(n_dof, dtype=vv.dtype)
-            vals[:gnods[0].shape[0] * dpn] = vv
+            vals[:gnods[0].shape[0] * dpn] = vv.ravel()
 
         else:
-            raise NotImplementedError
+            raise ValueError('unknown function/value type! (%s)' % type(fun))
 
         nods, indx = nm.unique(nods, return_index=True)
         ii = (nm.tile(dpn * indx, dpn)
@@ -231,7 +238,7 @@ class H1HierarchicVolumeField(H1Mixin, VolumeField):
         gps = self.gel.poly_space
         mesh = self.create_mesh(extra_nodes=False)
 
-        ctx = geo_ctx = gps.create_context(mesh.cmesh, 0, 1e-15, 100, 1e-8)
+        ctx = geo_ctx = gps.create_context(self.cmesh, 0, 1e-15, 100, 1e-8)
         ctx.geo_ctx = geo_ctx
 
         return ctx
