@@ -1,10 +1,11 @@
-from __future__ import absolute_import
+from itertools import product
+
 import numpy as nm
 import numpy.linalg as nla
 
 from sfepy.base.base import assert_, Struct
 from sfepy.discrete import PolySpace
-from sfepy.linalg import combine, insert_strided_axis
+from sfepy.linalg import insert_strided_axis
 from six.moves import range
 from functools import reduce
 
@@ -272,19 +273,19 @@ class LagrangePolySpace(FEPolySpace):
 
         return ctx
 
-    def _eval_base(self, coors, diff=0, ori=None,
-                   suppress_errors=False, eps=1e-15):
+    def _eval_basis(self, coors, diff=0, ori=None,
+                    suppress_errors=False, eps=1e-15):
         """
-        See :func:`PolySpace.eval_base()`.
+        See :func:`PolySpace.eval_basis()`.
         """
         if diff == 2:
-            base = self._eval_hessian(coors)
+            basis = self._eval_hessian(coors)
 
         else:
-            base = self.eval_ctx.evaluate(coors, diff=diff,
-                                          eps=eps,
-                                          check_errors=not suppress_errors)
-        return base
+            basis = self.eval_ctx.evaluate(coors, diff=diff,
+                                           eps=eps,
+                                           check_errors=not suppress_errors)
+        return basis
 
 class LagrangeSimplexPolySpace(LagrangePolySpace):
     """Lagrange polynomial space on a simplex domain."""
@@ -592,15 +593,15 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
 
         return nodes, nts, node_coors
 
-    def _eval_base_debug(self, coors, diff=False, ori=None,
-                         suppress_errors=False, eps=1e-15):
-        """Python version of eval_base()."""
+    def _eval_basis_debug(self, coors, diff=False, ori=None,
+                          suppress_errors=False, eps=1e-15):
+        """Python version of eval_basis()."""
         dim = self.geometry.dim
 
-        ev = self.ps1d.eval_base
+        ev = self.ps1d.eval_basis
 
         if diff:
-            base = nm.ones((coors.shape[0], dim, self.n_nod), dtype=nm.float64)
+            basis = nm.ones((coors.shape[0], dim, self.n_nod), dtype=nm.float64)
 
             for ii in range(dim):
                 self.ps1d.nodes = self.nodes[:,2*ii:2*ii+2].copy()
@@ -608,36 +609,36 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
 
                 for iv in range(dim):
                     if ii == iv:
-                        base[:,iv:iv+1,:] *= ev(coors[:,ii:ii+1].copy(),
-                                                diff=True,
-                                                suppress_errors=suppress_errors,
-                                                eps=eps)
+                        basis[:,iv:iv+1,:] *= ev(coors[:,ii:ii+1].copy(),
+                                                 diff=True,
+                                                 suppress_errors=suppress_errors,
+                                                 eps=eps)
 
                     else:
-                        base[:,iv:iv+1,:] *= ev(coors[:,ii:ii+1].copy(),
-                                                diff=False,
-                                                suppress_errors=suppress_errors,
-                                                eps=eps)
+                        basis[:,iv:iv+1,:] *= ev(coors[:,ii:ii+1].copy(),
+                                                 diff=False,
+                                                 suppress_errors=suppress_errors,
+                                                 eps=eps)
 
         else:
-            base = nm.ones((coors.shape[0], 1, self.n_nod), dtype=nm.float64)
+            basis = nm.ones((coors.shape[0], 1, self.n_nod), dtype=nm.float64)
 
             for ii in range(dim):
                 self.ps1d.nodes = self.nodes[:,2*ii:2*ii+2].copy()
                 self.ps1d.n_nod = self.n_nod
 
-                base *= ev(coors[:,ii:ii+1].copy(),
-                           diff=diff,
-                           suppress_errors=suppress_errors,
-                           eps=eps)
+                basis *= ev(coors[:,ii:ii+1].copy(),
+                            diff=diff,
+                            suppress_errors=suppress_errors,
+                            eps=eps)
 
-        return base
+        return basis
 
     def _eval_hessian(self, coors):
         """
         Evaluate the second derivatives of the basis.
         """
-        evh = self.ps1d.eval_base
+        evh = self.ps1d.eval_basis
 
         dim = self.geometry.dim
         bfgg = nm.zeros((coors.shape[0], dim, dim, self.n_nod),
@@ -678,6 +679,86 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
     def get_mtx_i(self):
         return self.ps1d.mtx_i
 
+
+class LagrangeWedgePolySpace(FEPolySpace):
+    """
+    """
+    name = 'lagrange_wedge'
+
+    def __init__(self, name, geometry, order, init_context=True):
+        from sfepy.discrete.fem.geometry_element import GeometryElement
+
+        PolySpace.__init__(self, name, geometry, order)
+
+        geom_1_2 = GeometryElement('1_2')
+        geom_2_3 = GeometryElement('2_3')
+
+        ps1 = LagrangeTensorProductPolySpace(f'{name}_1', geom_1_2, order,
+                                            #  init_context=False)
+                                             init_context=init_context)
+        ps2 = LagrangeSimplexPolySpace(f'{name}_2', geom_2_3, order,
+                                       init_context=init_context)
+                                    #    init_context=False)
+
+        geom_3_8 = GeometryElement('3_8')
+        ps0 = LagrangeTensorProductPolySpace(f'{name}_0', geom_3_8, order,
+                                             init_context=False)
+        geom_3_4 = GeometryElement('3_4')
+        ps0b = LagrangeSimplexPolySpace(f'{name}_0b', geom_3_4, order,
+                                        init_context=False)
+
+        n_nod = ps2.n_nod * ps1.n_nod
+        nd2 = ps2.nodes.shape[1]
+        nd = nd2 + ps1.nodes.shape[1]
+        self.nodes = nm.empty((n_nod, nd), nm.int32)
+        self.nodes[:, :nd2] = nm.tile(ps2.nodes, (ps1.n_nod, 1))
+        self.nodes[:, nd2:] = nm.repeat(ps1.nodes, ps2.n_nod, axis=0)
+        self.nts = nm.vstack([ps2.nts, ps1.nts])
+        self.nts[ps2.n_nod:, 1] += ps2.n_nod
+
+        self.node_coors = nm.empty((n_nod, 3), nm.int32)
+        self.node_coors[:, :2] = nm.tile(ps2.node_coors, (ps1.n_nod, 1))
+        self.node_coors[:, 2] = nm.repeat(ps1.node_coors, ps2.n_nod,
+                                          axis=0)[:, 0]
+
+        self.ps = [ps2, ps1]
+        self.n_nod = n_nod
+        self.eval_ctx = None
+
+    def _eval_basis(self, coors, diff=False, ori=None,
+                    suppress_errors=False, eps=1e-15):
+
+        nd = self.geometry.dim if diff else 1
+        basis = nm.empty((coors.shape[0], nd, self.n_nod), nm.float64)
+
+        for qp1 in nm.unique(coors[:, 2]):
+            idxs = coors[:, 2] == qp1
+            coors2 = coors[idxs, :2]
+            basis2 = self.ps[0]._eval_basis(coors2, False, ori,
+                                            suppress_errors, eps)
+            coors1 = coors[idxs, 2][:, None]
+            basis1 = self.ps[1]._eval_basis(coors1, False, ori,
+                                            suppress_errors, eps)
+
+            if diff:
+                basis2d = self.ps[0]._eval_basis(coors2, True, ori,
+                                                 suppress_errors, eps)
+                basis1d = self.ps[1]._eval_basis(coors1, True, ori,
+                                                 suppress_errors, eps)
+                basis_r = basis2d[:, 0, None, :] * basis1[:, 0, :, None]
+                basis_s = basis2d[:, 1, None, :] * basis1[:, 0, :, None]
+                basis_t = basis2[:, 0, None, :] * basis1d[:, 0, :, None]
+
+                basis[idxs, 0] = basis_r.reshape((-1, self.n_nod))
+                basis[idxs, 1] = basis_s.reshape((-1, self.n_nod))
+                basis[idxs, 2] = basis_t.reshape((-1, self.n_nod))
+
+            else:
+                basis_ = basis2[:, 0, None, :] * basis1[:, 0, :, None]
+                basis[idxs] = basis_.reshape(-1, 1, self.n_nod)
+
+        return basis
+
 class SerendipityTensorProductPolySpace(FEPolySpace):
     """
     Serendipity polynomial space using Lagrange functions.
@@ -786,10 +867,10 @@ class SerendipityTensorProductPolySpace(FEPolySpace):
 
         return nodes, nts, nm.ascontiguousarray(node_coors)
 
-    def _eval_base(self, coors, diff=0, ori=None,
-                   suppress_errors=False, eps=1e-15):
+    def _eval_basis(self, coors, diff=0, ori=None,
+                    suppress_errors=False, eps=1e-15):
         """
-        See :func:`PolySpace.eval_base()`.
+        See :func:`PolySpace.eval_basis()`.
         """
         dim = self.geometry.dim
         if diff:
@@ -798,21 +879,21 @@ class SerendipityTensorProductPolySpace(FEPolySpace):
         else:
             bdim = 1
 
-        base = nm.empty((coors.shape[0], bdim, self.n_nod), dtype=nm.float64)
+        basis = nm.empty((coors.shape[0], bdim, self.n_nod), dtype=nm.float64)
 
         if diff == 0:
             for ib, bf in enumerate(self._bfs):
-                base[:, 0, ib] = bf(*coors.T)
+                basis[:, 0, ib] = bf(*coors.T)
 
         elif diff == 1:
             for ib, bfg in enumerate(self._bfgs):
                 for ig in range(dim):
-                    base[:, ig, ib] = bfg[ig](*coors.T)
+                    basis[:, ig, ib] = bfg[ig](*coors.T)
 
         else:
             raise NotImplementedError
 
-        return base
+        return basis
 
 class LobattoTensorProductPolySpace(FEPolySpace):
     """
@@ -927,7 +1008,7 @@ class LobattoTensorProductPolySpace(FEPolySpace):
         # Bubble nodes.
         if n_bubble > 0:
             ik = nm.arange(2, order + 1, dtype=nm.int32)
-            nodes[ii:] = nm.array([aux for aux in combine([ik] * dim)])
+            nodes[ii:] = nm.array([aux for aux in product(*([ik] * dim))])
             nts[ii:ii + n_bubble] = [[3, 0]]
             ii += n_bubble
 
@@ -963,24 +1044,24 @@ class LobattoTensorProductPolySpace(FEPolySpace):
 
         return anodes
 
-    def _eval_base(self, coors, diff=False, ori=None,
-                   suppress_errors=False, eps=1e-15):
+    def _eval_basis(self, coors, diff=False, ori=None,
+                    suppress_errors=False, eps=1e-15):
         """
-        See PolySpace.eval_base().
+        See PolySpace.eval_basis().
         """
         from .extmods.lobatto_bases import eval_lobatto_tensor_product as ev
         c_min, c_max = self.bbox[:, 0]
 
-        base = ev(coors, self.nodes, c_min, c_max, self.order, diff)
+        basis = ev(coors, self.nodes, c_min, c_max, self.order, diff)
 
         if ori is not None:
-            ebase = nm.tile(base, (ori.shape[0], 1, 1, 1))
+            ebasis = nm.tile(basis, (ori.shape[0], 1, 1, 1))
 
             if self.edge_indx.shape[0]:
                 # Orient edge functions.
                 ie, ii = nm.where(ori[:, self.edge_indx] == 1)
                 ii = self.edge_indx[ii]
-                ebase[ie, :, :, ii] *= -1.0
+                ebasis[ie, :, :, ii] *= -1.0
 
             if self.face_indx.shape[0]:
                 # Orient face functions.
@@ -989,22 +1070,282 @@ class LobattoTensorProductPolySpace(FEPolySpace):
                 # ... normal axis order
                 ie, ii = nm.where((fori == 1) | (fori == 2))
                 ii = self.face_indx[ii]
-                ebase[ie, :, :, ii] *= -1.0
+                ebasis[ie, :, :, ii] *= -1.0
 
                 # ... swapped axis order
-                sbase = ev(coors, self.sfnodes, c_min, c_max, self.order, diff)
-                sbase = insert_strided_axis(sbase, 0, ori.shape[0])
+                sbasis = ev(coors, self.sfnodes, c_min, c_max, self.order, diff)
+                sbasis = insert_strided_axis(sbasis, 0, ori.shape[0])
 
                 # ...overwrite with swapped axes basis.
                 ie, ii = nm.where(fori >= 4)
                 ii2 = self.face_indx[ii]
-                ebase[ie, :, :, ii2] = sbase[ie, :, :, ii]
+                ebasis[ie, :, :, ii2] = sbasis[ie, :, :, ii]
 
                 # ...deal with orientation.
                 ie, ii = nm.where((fori == 5) | (fori == 6))
                 ii = self.face_indx[ii]
-                ebase[ie, :, :, ii] *= -1.0
+                ebasis[ie, :, :, ii] *= -1.0
 
-            base = ebase
+            basis = ebasis
 
-        return base
+        return basis
+
+class BernsteinSimplexPolySpace(FEPolySpace):
+    """
+    Bernstein polynomial space on simplex domains.
+
+    Notes
+    -----
+    Naive proof-of-concept implementation, does not use recurrent formulas or
+    Duffy transformation to obtain tensor product structure.
+    """
+    name = 'bernstein_simplex'
+
+    def __init__(self, name, geometry, order):
+        PolySpace.__init__(self, name, geometry, order)
+
+        self.nodes, self.nts, self.node_coors = self._define_nodes()
+        self.n_nod = self.nodes.shape[0]
+        self.eval_ctx = None
+
+    def _define_nodes(self):
+        nodes, nts, node_coors = LagrangeSimplexPolySpace._define_nodes(self)
+        return nodes, nts, node_coors
+
+    @staticmethod
+    def _get_barycentric(coors):
+        dim = coors.shape[1]
+
+        bcoors = nm.empty((coors.shape[0], dim + 1))
+        bcoors[:, 0] = 1.0 - coors.sum(axis=1)
+        bcoors[:, 1:] = coors
+        return bcoors
+
+    def _eval_basis(self, coors, diff=False, ori=None,
+                    suppress_errors=False, eps=1e-15):
+        """
+        See PolySpace.eval_basis().
+        """
+        from scipy.special import factorial
+
+        dim = self.geometry.dim
+        if diff:
+            bdim = dim
+
+            bgrad = nm.zeros((dim + 1, dim), dtype=nm.float64)
+            bgrad[0] = -1
+            bgrad[1:] = nm.eye(dim)
+
+        else:
+            bdim = 1
+
+        basis = nm.ones((coors.shape[0], bdim, self.n_nod), dtype=nm.float64)
+        if dim == 0:
+            return basis
+
+        bcoors = self._get_barycentric(coors)
+
+        fs = factorial(nm.arange(0, self.order + 1))
+        of = fs[-1]
+        if not diff:
+            for ii, node in enumerate(self.nodes):
+                coef = of / nm.prod(fs[node])
+                val = coef * nm.prod(nm.power(bcoors, node), axis=1)
+                basis[:, 0, ii] = val
+
+        else:
+            for ii, node in enumerate(self.nodes):
+                coef = of / nm.prod(fs[node])
+                for ider in range(dim):
+                    dval = 0.0
+                    for ib in range(dim + 1):
+                        ex = node[ib]
+                        val = coef
+                        for im in range(dim + 1):
+                            if ib == im:
+                                val *= (ex *
+                                        nm.power(bcoors[:, im], ex - 1) *
+                                        bgrad[ib, ider])
+
+                            else:
+                                val *= nm.power(bcoors[:, im], node[im])
+
+                        dval += val
+
+                    basis[:, ider, ii] = dval
+
+        return basis
+
+class BernsteinTensorProductPolySpace(FEPolySpace):
+    """
+    Bernstein polynomial space.
+
+    Each row of the `nodes` attribute defines indices of 1D Bernstein basis
+    functions that need to be multiplied together to evaluate the corresponding
+    shape function. This defines the ordering of basis functions on the
+    reference element.
+    """
+    name = 'bernstein_tensor_product'
+    def __init__(self, name, geometry, order):
+        PolySpace.__init__(self, name, geometry, order)
+
+        self.nodes, self.nts, self.node_coors = self._define_nodes()
+        self.n_nod = self.nodes.shape[0]
+        self.eval_ctx = None
+
+    def _define_nodes(self):
+        nn, nts, node_coors = LagrangeTensorProductPolySpace._define_nodes(self)
+        nodes = nn[:, 1::2]
+
+        return nodes, nts, node_coors
+
+    def _eval_basis(self, coors, diff=False, ori=None,
+                    suppress_errors=False, eps=1e-15):
+        """
+        See PolySpace.eval_basis().
+        """
+        from sfepy.discrete.iga.extmods.igac import eval_bernstein_basis as ev
+
+        dim = self.geometry.dim
+        if diff:
+            bdim = dim
+
+        else:
+            bdim = 1
+
+        basis = nm.ones((coors.shape[0], bdim, self.n_nod), dtype=nm.float64)
+        degree = self.order
+        n_efuns_max = degree + 1
+
+        for iq, qp in enumerate(coors):
+            B = nm.empty((dim, n_efuns_max), dtype=nm.float64)
+            dB_dxi = nm.empty((dim, n_efuns_max), dtype=nm.float64)
+            for ii in range(dim):
+                ev(B[ii, :], dB_dxi[ii, :], qp[ii], degree)
+
+            if not diff:
+                for ii, ni in enumerate(self.nodes.T):
+                    basis[iq, 0, :] *= B[ii, ni]
+
+            else:
+                for ii, ni in enumerate(self.nodes.T):
+                    for iv in range(bdim):
+                        if ii == iv:
+                            basis[iq, iv, :] *= dB_dxi[ii, ni]
+
+                        else:
+                            basis[iq, iv, :] *= B[ii, ni]
+
+        return basis
+
+def get_lgl_nodes(p):
+    """
+    Compute the Legendre-Gauss-Lobatto nodes and weights.
+    """
+    from numpy.polynomial.legendre import legvander
+
+    # Use the Chebyshev-Gauss-Lobatto nodes as the first guess.
+    xs = nm.cos(nm.pi * nm.arange(p + 1) / p)
+
+    eps = nm.finfo(nm.float64).eps
+    xs0 = 2.0
+    while nm.linalg.norm(xs - xs0, ord=nm.inf) > eps:
+        xs0 = xs
+        V = legvander(xs, p)
+        xs = xs0 - (xs * V[:, p] - V[:,p-1]) / ((p+1) * V[:,p])
+
+    ws = 2.0 / (p * (p+1) * V[:,p]**2)
+
+    return xs, ws
+
+def eval_lagrange1d_basis(coors, ncoors):
+    n_nod = len(ncoors)
+    n_coors = len(coors)
+
+    val = nm.ones((n_coors, n_nod), dtype=nm.float64)
+    dval = nm.zeros((n_coors, n_nod), dtype=nm.float64)
+    for ib in range(n_nod):
+        for ic in range(n_nod):
+            if ib != ic:
+                val[:, ib] *= ((coors - ncoors[ic])
+                               / (ncoors[ib] - ncoors[ic]))
+
+        for ik in range(n_nod):
+            if ib == ik: continue
+            aux = 1.0 / (ncoors[ib] - ncoors[ik])
+            for ic in range(n_nod):
+                if (ib != ic) and (ik != ic):
+                    aux *= ((coors - ncoors[ic])
+                            / (ncoors[ib] - ncoors[ic]))
+
+            dval[:, ib] += aux
+
+    return val, dval
+
+class SEMTensorProductPolySpace(FEPolySpace):
+    """
+    Spectral element method polynomial space = Lagrange polynomial space with
+    Legendre-Gauss-Lobatto nodes. The same nodes and corresponding weights
+    should be used for numerical quadrature to obtain a diagonal mass matrix.
+    """
+    name = 'sem_tensor_product'
+
+    def __init__(self, name, geometry, order, init_context=True):
+        PolySpace.__init__(self, name, geometry, order)
+
+        (self.nodes, self.nts,
+         node_coors, self.node_weights,
+         self.node_coors1d, self.weights1d) = self._define_nodes()
+        self.node_coors = nm.ascontiguousarray(node_coors)
+
+        self.n_nod = self.nodes.shape[0]
+        self.eval_ctx = None
+
+    def _define_nodes(self):
+        nn, nts, node_coors = LagrangeTensorProductPolySpace._define_nodes(self)
+        nodes = nn[:, 1::2]
+
+        node_coors1d, weights1d = get_lgl_nodes(self.order)
+        # Transform node_coors1d from [1, -1] to [0, 1].
+        node_coors1d = 0.5 * (1 - node_coors1d)
+        weights1d *= 0.5
+
+        node_weights = nm.ones_like(node_coors[:, 0])
+        for ii, ni in enumerate(nodes.T):
+            node_coors[:, ii] = node_coors1d[ni]
+            node_weights[:] *= weights1d[ni]
+
+        return nodes, nts, node_coors, node_weights, node_coors1d, weights1d
+
+    def _eval_basis(self, coors, diff=0, ori=None,
+                    suppress_errors=False, eps=1e-15):
+        """
+        See :func:`PolySpace.eval_basis()`.
+        """
+        dim = self.geometry.dim
+        bdim = dim if diff else 1
+
+        assert diff in (0, 1)
+
+        out = nm.ones((coors.shape[0], bdim, self.n_nod), dtype=nm.float64)
+        vals = []
+        dvals = []
+        for ii in range(dim):
+            b1d, db1d = eval_lagrange1d_basis(coors[:, ii], self.node_coors1d)
+            vals.append(b1d)
+            dvals.append(db1d)
+
+        if diff == 0:
+            for ii, ni in enumerate(self.nodes.T):
+                out[:, 0, :] *= vals[ii][:, ni]
+
+        else:
+            for ii, ni in enumerate(self.nodes.T):
+                for iv in range(bdim):
+                    if ii == iv:
+                        out[:, iv, :] *= dvals[ii][:, ni]
+
+                    else:
+                        out[:, iv, :] *= vals[ii][:, ni]
+
+        return out
