@@ -1,9 +1,13 @@
 """Classes for probing values of Variables, for example, along a line."""
 from __future__ import absolute_import
+import hashlib
+
+from ast import literal_eval
+
 import numpy as nm
 import numpy.linalg as nla
 
-from sfepy.base.base import get_default, basestr, Struct
+from sfepy.base.base import assert_, basestr, Struct
 from sfepy.linalg import make_axis_rotation_matrix, norm_l2_along_axis
 import six
 
@@ -26,13 +30,10 @@ def write_results(filename, probe, results):
     fd.write('\n'.join(probe.report()) + '\n')
     for key, result in six.iteritems(results):
         pars, vals = result
+        vals = vals.reshape(pars.shape[0], -1)
         fd.write('\n# %s %d\n' % (key, vals.shape[-1]))
 
-        if vals.ndim == 1:
-            aux = nm.hstack((pars[:,None], vals[:,None]))
-
-        else:
-            aux = nm.hstack((pars[:,None], vals))
+        aux = nm.concatenate((pars[:, None], vals), axis=1)
 
         nm.savetxt(fd, aux)
 
@@ -58,14 +59,14 @@ def read_results(filename, only_names=None):
     """
     from sfepy.base.ioutils import read_array
 
-    only_names = get_default(only_names, [])
+    is_only_names = only_names is not None
 
     fd = open(filename, 'r') if isinstance(filename, basestr) else filename
 
     header = read_header(fd)
     results = {}
     for name, nc in get_data_name(fd):
-        if name not in only_names: continue
+        if is_only_names and (name not in only_names): continue
 
         result = read_array(fd, header.n_point, nc + 1, nm.float64)
         results[name] = result
@@ -95,7 +96,9 @@ def read_header(fd):
             break
         else:
             details.append(line)
-    header.details = '\n'.join(details)
+
+    cls = globals()[header.probe_class]
+    header.details = cls.parse_report(details)
 
     return header
 
@@ -117,7 +120,7 @@ def get_data_name(fd):
             if (len(line) == 0): break
             if len(line) == 1: continue
         except:
-            raise StopIteration
+            return
 
         line = line.strip().split()
         if (len(line) == 3) and (line[0] == '#'):
@@ -126,6 +129,16 @@ def get_data_name(fd):
 
             yield name, nc
 
+def parse_vector(line):
+    svec = line.split(':')[1].strip('[] ')
+    vec = nm.fromstring(svec, dtype=nm.float64, sep=' ')
+    return vec
+
+def parse_scalar(line):
+    sval = line.split(':')[1].strip('[] ')
+    val = literal_eval(sval)
+    return val
+
 class Probe(Struct):
     """
     Base class for all point probes. Enforces two points minimum.
@@ -159,6 +172,8 @@ class Probe(Struct):
 
         self.options = Struct(close_limit=0.1, size_hint=None)
         self.cache = Struct(name='probe_local_evaluate_cache')
+        self.acache = Struct(name='probe_actual_evaluate_cache',
+                             pars_digest='')
 
         self.is_refined = False
 
@@ -169,6 +184,34 @@ class Probe(Struct):
         """
         return Probe.cache if self.share_geometry else self.cache
 
+    def get_actual_cache(self, pars, cache, hash_chunk_size=100000):
+        """
+        Return the actual evaluate cache, which is a combination of the
+        (mesh-based) evaluate cache and probe-specific data, like the reference
+        element coordinates. The reference element coordinates are reused, if
+        the sha1 hash of the probe parameter vector does not change.
+        """
+        self.acache += cache
+
+        def _gen_array_chunks(arr):
+            ii = 0
+            while len(arr[ii:]):
+                yield arr[ii:ii+hash_chunk_size].tobytes()
+                ii += hash_chunk_size
+
+        sha1 = hashlib.sha1()
+        for chunk in _gen_array_chunks(pars):
+            sha1.update(chunk)
+
+        digest = sha1.hexdigest()
+        if digest != self.acache.pars_digest:
+            self.acache.pars_digest = digest
+            self.acache.ref_coors = None
+            self.acache.cells = None
+            self.acache.status = None
+
+        return self.acache
+
     def set_n_point(self, n_point):
         """
         Set the number of probe points.
@@ -276,9 +319,16 @@ class Probe(Struct):
             if not nm.isfinite(points).all():
                 raise ValueError('Inf/nan in probe points!')
 
-            vals, cells = ev(points, mode=mode, strategy='general',
-                             close_limit=self.options.close_limit,
-                             cache=cache, ret_cells=True)
+            acache = self.get_actual_cache(pars, cache)
+
+            vals, ref_coors, cells, status = ev(
+                points, mode=mode, strategy='general',
+                close_limit=self.options.close_limit, cache=acache,
+                ret_ref_coors=True, ret_status=True, ret_cells=True)
+
+            acache.ref_coors = ref_coors
+            acache.cells = cells
+            acache.status = status
 
             if self.is_refined:
                 break
@@ -386,6 +436,15 @@ class PointsProbe(Probe):
         out.append('-----')
         return out
 
+    @staticmethod
+    def parse_report(lines):
+        """
+        Parse report lines to get the probe parameters.
+        """
+        out = [parse_vector(line) for line in lines]
+
+        return out
+
     def refine_points(self, variable, points, cache):
         """No refinement for this probe."""
         refine_flag = nm.array([False])
@@ -443,6 +502,16 @@ class LineProbe(Probe):
         out.append('-----')
         return out
 
+    @staticmethod
+    def parse_report(lines):
+        """
+        Parse report lines to get the probe parameters.
+        """
+        out = [parse_vector(line) for line in lines]
+        assert_(len(out) == 2)
+
+        return out
+
     def get_points(self, refine_flag=None):
         """
         Get the probe points.
@@ -519,6 +588,16 @@ class RayProbe(Probe):
         out.append('-----')
         return out
 
+    @staticmethod
+    def parse_report(lines):
+        """
+        Parse report lines to get the probe parameters.
+        """
+        out = [parse_vector(lines[0]), parse_vector(lines[1]),
+               parse_scalar(lines[2]), lines[3]]
+
+        return out
+
     def refine_points(self, variable, points, cache):
         """No refinement for this probe."""
         refine_flag = nm.array([False])
@@ -589,6 +668,16 @@ class CircleProbe(Probe):
         out.append('-----')
         return out
 
+    @staticmethod
+    def parse_report(lines):
+        """
+        Parse report lines to get the probe parameters.
+        """
+        out = [parse_vector(lines[0]), parse_vector(lines[1]),
+               parse_scalar(lines[2])]
+
+        return out
+
     def get_points(self, refine_flag=None):
         """
         Get the probe points.
