@@ -55,7 +55,6 @@ import numpy as nm
 
 from sfepy.base.base import output, assert_, Struct
 from sfepy.discrete.simplex_cubature import get_simplex_cubature
-import six
 
 simplex_geometries = ['1_2', '2_3', '3_4']
 tp_geometries = ['2_4', '3_8']
@@ -128,7 +127,7 @@ class QuadraturePoints(Struct):
     """
 
     @staticmethod
-    def from_table(geometry, order):
+    def from_table(geometry, order, full_order=False):
         """
         Create a new :class:`QuadraturePoints` instance, given reference
         element geometry name and polynomial order. For tensor product
@@ -154,10 +153,11 @@ class QuadraturePoints(Struct):
             qp.order = order
 
         else:
-            order1d = order
-            dim = int(geometry[0])
+            if not full_order:
+                order1d = order
+                dim = int(geometry[0])
+                order = dim * order1d
 
-            order = dim * order1d
             if order <= max_orders[geometry]:
                 order = get_actual_order(geometry, order)
                 qp = table[order]
@@ -789,6 +789,37 @@ quadrature_tables = {
 
     },
 
+    '3_6': {
+        3: _QP([[1.0/3.0, 1.0/3.0, 0.5, 0.5]],
+               tp_fix=1/2),
+        6: _QP([[1.0/6.0, 1.0/6.0, 0.2113248654051871, 1.0/12.0],
+                [2.0/3.0, 1.0/6.0, 0.2113248654051871, 1.0/12.0],
+                [1.0/6.0, 2.0/3.0, 0.2113248654051871, 1.0/12.0],
+                [1.0/6.0, 1.0/6.0, 0.7886751345948129, 1.0/12.0],
+                [2.0/3.0, 1.0/6.0, 0.7886751345948129, 1.0/12.0],
+                [1.0/6.0, 2.0/3.0, 0.7886751345948129, 1.0/12.0]],
+               tp_fix=1/2),
+        9: _QP([[0.1081030181680702, 0.4459484909159649, 0.1127016653792583, 0.0310252207886127],
+                [0.4459484909159649, 0.1081030181680702, 0.1127016653792583, 0.0310252207886127],
+                [0.4459484909159649, 0.4459484909159649, 0.1127016653792583, 0.0310252207886127],
+                [0.8168475729804585, 0.0915762135097707, 0.1127016653792583, 0.0152710755076836],
+                [0.0915762135097707, 0.8168475729804585, 0.1127016653792583, 0.0152710755076836],
+                [0.0915762135097707, 0.0915762135097707, 0.1127016653792583, 0.0152710755076836],
+                [0.1081030181680702, 0.4459484909159649, 0.5000000000000000, 0.0496403532617803],
+                [0.4459484909159649, 0.1081030181680702, 0.5000000000000000, 0.0496403532617803],
+                [0.4459484909159649, 0.4459484909159649, 0.5000000000000000, 0.0496403532617803],
+                [0.8168475729804585, 0.0915762135097707, 0.5000000000000000, 0.0244337208122937],
+                [0.0915762135097707, 0.8168475729804585, 0.5000000000000000, 0.0244337208122937],
+                [0.0915762135097707, 0.0915762135097707, 0.5000000000000000, 0.0244337208122937],
+                [0.1081030181680702, 0.4459484909159649, 0.8872983346207417, 0.0310252207886127],
+                [0.4459484909159649, 0.1081030181680702, 0.8872983346207417, 0.0310252207886127],
+                [0.4459484909159649, 0.4459484909159649, 0.8872983346207417, 0.0310252207886127],
+                [0.8168475729804585, 0.0915762135097707, 0.8872983346207417, 0.0152710755076836],
+                [0.0915762135097707, 0.8168475729804585, 0.8872983346207417, 0.0152710755076836],
+                [0.0915762135097707, 0.0915762135097707, 0.8872983346207417, 0.0152710755076836]],
+               tp_fix=1/2),
+    },
+
     '3_8' : {
 
         2 : _QP([[ 0.0             , nm.sqrt(2.0/3.0),-1/nm.sqrt(3), 2.0],
@@ -824,7 +855,7 @@ del _QP
 
 def _get_max_orders():
     max_orders = {}
-    for key, table in six.iteritems(quadrature_tables):
+    for key, table in quadrature_tables.items():
         orders = list(table.keys())
         max_orders[key] = max(orders)
 
