@@ -1,4 +1,5 @@
 from __future__ import absolute_import, division
+import gc
 from copy import copy
 
 from sfepy.base.base import output, get_default, Struct
@@ -103,6 +104,7 @@ class HomogenizationWorker(object):
                                      time_tag)
 
             dependencies[name] = val
+            gc.collect()
 
         return dependencies, save_names
 
@@ -165,7 +167,7 @@ class HomogenizationWorker(object):
             val = []
             if hasattr(mini_app, 'store_idxs') and mode == 'reqs':
                 save_name = mini_app.save_name
-            
+
             local_coors = local_state['coors']
             for im in range(len(local_coors)):
                 output('== micro %s%s-%d =='
@@ -196,6 +198,9 @@ class HomogenizationWorker(object):
 
                     val.append(mini_app(data=get_dict_idxval(data, im)))
 
+                    if len(val) == 1 and val[0].name == 'update_coors':
+                        local_coors[im] += val[0].state
+
         return val
 
     @staticmethod
@@ -267,7 +272,7 @@ class HomogenizationWorker(object):
             mini_app = MiniAppBase.any_from_conf(name, problem, rargs)
             mini_app.setup_output(save_formats=opts.save_formats,
                                   post_process_hook=post_process_hook,
-                                  file_per_var=opts.file_per_var)
+                                  split_results_by=opts.split_results_by)
             if mini_app.save_name is not None:
                 mini_app.save_name += time_tag
 
