@@ -2,7 +2,7 @@ from __future__ import absolute_import
 import numpy as nm
 import numpy.linalg as nla
 
-from scipy.misc import factorial
+from scipy.special import factorial
 
 from sfepy.base.base import assert_, output
 from sfepy.linalg.utils import norm_l2_along_axis as norm
@@ -172,7 +172,7 @@ def points_in_simplex(coors, s_coors, eps=1e-8):
     """
     n_c, dim = coors.shape
     bc = barycentric_coors(coors, s_coors)
-    flag = nm.ones((n_c,), dtype=nm.bool)
+    flag = nm.ones((n_c,), dtype=bool)
     for idim in range(dim + 1):
         flag &= nm.where((bc[idim,:] > -eps)
                          & (bc[idim,:] < (1.0 + eps)), True, False)
@@ -200,7 +200,7 @@ def flag_points_in_polygon2d(polygon, coors):
 
     [1] PNPOLY - Point Inclusion in Polygon Test, W. Randolph Franklin (WRF)
     """
-    flag = nm.zeros(coors.shape[0], dtype=nm.bool)
+    flag = nm.zeros(coors.shape[0], dtype=bool)
     nv = polygon.shape[0]
     px, py = coors[:, 0], coors[:, 1]
     for ii in range(nv):
@@ -212,7 +212,7 @@ def flag_points_in_polygon2d(polygon, coors):
                              ~flag[aux], flag[aux])
     return flag
 
-def inverse_element_mapping(coors, e_coors, eval_base, ref_coors,
+def inverse_element_mapping(coors, e_coors, eval_basis, ref_coors,
                             suppress_errors=False):
     """
     Given spatial element coordinates, find the inverse mapping for
@@ -237,14 +237,14 @@ def inverse_element_mapping(coors, e_coors, eval_base, ref_coors,
 
     else: # Tensor-product and other.
         def residual(xi):
-            bf = eval_base(xi[nm.newaxis,:].copy(),
-                           suppress_errors=suppress_errors).squeeze()
+            bf = eval_basis(xi[nm.newaxis,:].copy(),
+                            suppress_errors=suppress_errors).squeeze()
             res = coors - nm.dot(bf, e_coors)
             return res.squeeze()
 
         def matrix(xi):
-            bfg = eval_base(xi[nm.newaxis,:].copy(), diff=True,
-                            suppress_errors=suppress_errors).squeeze()
+            bfg = eval_basis(xi[nm.newaxis,:].copy(), diff=True,
+                             suppress_errors=suppress_errors).squeeze()
             mtx = - nm.dot(bfg, e_coors)
             return mtx
 
@@ -306,13 +306,13 @@ def get_face_areas(faces, coors):
 
     if n_v == 3:
         aux = coors[faces]
+        if aux.shape[-1] == 2:
+            zz = nm.zeros(aux.shape[:-1] + (1,))
+            aux = nm.concatenate((aux, zz), axis=-1)
+
         v1 = aux[:, 1, :] - aux[:, 0, :]
         v2 = aux[:, 2, :] - aux[:, 0, :]
-        if coors.shape[1] == 3:
-            areas = 0.5 * norm(nm.cross(v1, v2))
-
-        else:
-            areas = 0.5 * nm.abs(nm.cross(v1, v2))
+        areas = 0.5 * norm(nm.cross(v1, v2))
 
     elif n_v == 4:
         areas1 = get_face_areas(faces[:, [0, 1, 2]], coors)
@@ -437,10 +437,16 @@ def get_coors_in_tube(coors, centre, axis, radius_in, radius_out, length,
 
     return out
 
-def get_coors_in_ball(coors, centre, radius, inside=True):
+def get_coors_in_ball(coors, centre, radius, radius2=None, inside=True):
     """
     Return indices of coordinates inside or outside a ball given by
-    centre and radius.
+    centre and radius::
+
+      inside radius radius2 condition
+      True   r      None          |x - c| <= r
+      True   r      r2      r2 <= |x - c| <= r
+      False  r      None    |x - c| >= r
+      False  r      r2      |x - c| >= r | |x - c| <= r2
 
     Notes
     -----
@@ -451,11 +457,19 @@ def get_coors_in_ball(coors, centre, radius, inside=True):
     centre = nm.asarray(centre)
 
     vec = coors - centre[None, :]
+    nvec = norm(vec)
+    if radius2 is None:
+        if inside:
+            out = nm.where(nvec <= radius)[0]
 
-    if inside:
-        out = nm.where(norm(vec) <= radius)[0]
+        else:
+            out = nm.where(nvec >= radius)[0]
 
     else:
-        out = nm.where(norm(vec) >= radius)[0]
+        if inside:
+            out = nm.where((nvec <= radius) & (nvec >= radius2))[0]
+
+        else:
+            out = nm.where((nvec >= radius) & (nvec <= radius2))[0]
 
     return out
