@@ -146,8 +146,50 @@ def youngpoisson_from_stiffness(stiffness, plane='strain'):
 
     return young, poisson
 
-elastic_constants_relations = {
-}
+
+def stiffness_from_yps_ortho3(young, poisson, shear):
+    r"""
+    Compute 3D stiffness tensor :math:`{\bm D}` of an orthotropic linear
+    elastic material. Young's modulus (:math:`[E_1, E_2, E_3]`),
+    Poisson's ratio (:math:`[\nu_{12}, \nu_{13}, \nu_{23}]`),
+    and shear modulus (:math:`[G_{12}, G_{13}, G_{23}]`) are given.
+
+    .. math::
+        {\bm C}_{(3D)} = \begin{bmatrix}
+        1/E_1 & -\nu_{21}/E_2 & -\nu_{31}/E_3 & 0 & 0 & 0\\
+        -\nu_{12}/E_1 & 1/E_2 & -\nu_{32}/E_3 & 0 & 0 & 0\\
+        -\nu_{13}/E_1 & -\nu_{23}/E_2 & 1/E_3 & 0 & 0 & 0\\
+        0 & 0 & 0 & 1/G_{12} & 0 & 0 \\
+        0 & 0 & 0 & 0 & 1/G_{13} & 0 \\
+        0 & 0 & 0 & 0 & 0 & 1/G_{23} \end{bmatrix}
+
+    .. math::
+        {\bm D}_{(3D)} = \mathrm{inv}({\bm C}_{(3D)})
+
+    .. math::
+        \nu_{21} = \nu_{12}\frac{E_2}{E_1},\quad
+        \nu_{31} = \nu_{13}\frac{E_3}{E_1},\quad
+        \nu_{32} = \nu_{23}\frac{E_3}{E_2}
+
+    [1] R.M. Jones: Mechanics of composite materials. 1999.
+    """
+    # symmetric order [11, 22, 33, 12, 13, 23]
+    compl = nm.zeros((6, 6), dtype=nm.float64)
+
+    compl[0, 0] = 1. / young[0]
+    compl[1, 1] = 1. / young[1]
+    compl[2, 2] = 1. / young[2]
+    compl[0, 1] = compl[1, 0] = -poisson[0] / young[0]
+    compl[0, 2] = compl[2, 0] = -poisson[1] / young[0]
+    compl[1, 2] = compl[2, 1] = -poisson[2] / young[1]
+    compl[3, 3] = 1. / shear[0]
+    compl[4, 4] = 1. / shear[1]
+    compl[5, 5] = 1. / shear[2]
+
+    stiff = nm.linalg.inv(compl)
+
+    return stiff
+
 
 class ElasticConstants(Struct):
     r"""
@@ -382,36 +424,102 @@ class TransformToPlane(Struct):
         """
         mg = nm.meshgrid
 
-        cs = c3[mg(self.i_ss, self.i_ss)]
-        cm = c3[mg(self.i_ss, self.i_ms)].T
+        cs = c3[tuple(mg(self.i_ss, self.i_ss))]
+        cm = c3[tuple(mg(self.i_ss, self.i_ms))].T
         if d3 is None: # elasticity only.
             A = cs
             Feps = cm
 
             Ainv = nm.linalg.inv(A)
-            c2 = c3[mg(self.i_ms, self.i_ms)] \
+            c2 = c3[tuple(mg(self.i_ms, self.i_ms))] \
                  - nm.dot(Feps.T, nm.dot(Ainv, Feps))
 
             return c2
 
         else:
-            dm = d3[mg(self.i_s, self.i_m)].T
-            ds = d3[mg(self.i_s, self.i_s)]
+            dm = d3[tuple(mg(self.i_s, self.i_m))].T
+            ds = d3[tuple(mg(self.i_s, self.i_s))]
 
-            ii = mg(self.i_s, self.i_ss)
+            ii = tuple(mg(self.i_s, self.i_ss))
             A = nm.r_[nm.c_[cs, b3[ii]],
                       nm.c_[b3[ii].T, -ds]] #=> sym !!!
-            F = nm.r_[nm.c_[cm, b3[mg(self.i_m, self.i_ss)]],
-                      nm.c_[b3[mg(self.i_s, self.i_ms)].T, -dm]]
+            F = nm.r_[nm.c_[cm, b3[tuple(mg(self.i_m, self.i_ss))]],
+                      nm.c_[b3[tuple(mg(self.i_s, self.i_ms))].T, -dm]]
 
             Feps = F[:, :3]
             FE = F[:, 3:]
             Ainv = nm.linalg.inv(A)
-            c2 = c3[mg(self.i_ms, self.i_ms)] \
+            c2 = c3[tuple(mg(self.i_ms, self.i_ms))] \
                  - nm.dot(Feps.T, nm.dot(Ainv, Feps))
-            d2 = d3[mg(self.i_m, self.i_m)] \
+            d2 = d3[tuple(mg(self.i_m, self.i_m))] \
                  - nm.dot(FE.T, nm.dot(Ainv, FE))
-            b2 = b3[mg(self.i_m, self.i_ms)].T \
+            b2 = b3[tuple(mg(self.i_m, self.i_ms))].T \
                  - nm.dot(FE.T, nm.dot(Ainv, Feps))
 
             return c2, d2, b2
+
+def youngpoisson_from_wave_speeds(vp, vs, rho):
+    r"""
+    Compute the Young's modulus :math:`E` and Poisson's ratio :math:`\nu` from
+    the P- and S-wave speeds in a homogeneous isotropic material.
+
+    .. math::
+       E = {\rho v_s^2 (3 v_p^2 - 4 v_s^2) \over (v_p^2 - v_s^2)}
+
+    .. math::
+       \nu = {(v_p^2/2 - v_s^2) \over (v_p^2 - v_s^2)}
+
+    Parameters
+    ----------
+    vp : float or array
+        The P-wave speed.
+    vs : float or array
+        The S-wave speed.
+    rho : float or array
+        The density.
+
+    Returns
+    -------
+    young : float or array
+        The Young's modulus.
+    poisson : float or array
+        The Poisson's ratio.
+    """
+    vs2 = vs**2
+    vp2 = vp**2
+    aux = vp2 - vs2
+    return (vs2 * rho * (3.0*vp2 - 4.0*vs**2)/aux,
+            (vp2/2.0 - vs2)/aux)
+
+def wave_speeds_from_youngpoisson(young, poisson, rho):
+    r"""
+    Compute the P- and S-wave speeds from the Young's modulus :math:`E` and
+    Poisson's ratio :math:`\nu` in a homogeneous isotropic material.
+
+    .. math::
+       v_p^2 = {E (1 - \nu) \over \rho (1 + \nu) (1 - 2 \nu)}
+             = {(\lambda + 2 \mu) \over \rho}
+
+    .. math::
+       v_s^2 = {E \over 2 \rho (1 + \nu)}
+             = {\mu \over \rho}
+
+    Parameters
+    ----------
+    young : float or array
+        The Young's modulus.
+    poisson : float or array
+        The Poisson's ratio.
+    rho : float or array
+        The density.
+
+    Returns
+    -------
+    vp : float or array
+        The P-wave speed.
+    vs : float or array
+        The S-wave speed.
+    """
+    return (nm.sqrt(young * (1.0 - poisson)
+                    / (rho * (1.0 + poisson) * (1.0 - 2.0*poisson))),
+            nm.sqrt(young / (2.0 * rho * (1.0 + poisson))))
