@@ -1,4 +1,9 @@
+from __future__ import print_function
+from __future__ import absolute_import
 import sys
+from six.moves import range
+sys.path.append('.')
+
 import numpy as nm
 from sfepy.base.base import Struct
 
@@ -11,11 +16,10 @@ nm_f64_eps = nm.finfo(nm.float64).eps
 def to_ndarray(a):
     if a is None:
         return None
-    elif nm.isscalar(a):
-        return nm.array([a])
-    elif type(a) is not nm.ndarray:
-        return nm.array(a)
     else:
+        a = nm.asarray(a)
+        if len(a.shape) == 0:
+            a = a.reshape(1)
         return a
 
 class BSpline(Struct):
@@ -225,7 +229,7 @@ class BSpline(Struct):
             The knot vector.
         """
         self.knot_type = 'userdef'
-        self.knots = knots
+        self.knots = to_ndarray(knots)
 
     def get_knot_vector(self):
         """
@@ -284,10 +288,10 @@ class BSpline(Struct):
                 self.eval_basis()
 
             else:
-                print 'knot insertion failed: multiplicity = spline degree!'
+                print('knot insertion failed: multiplicity = spline degree!')
 
         else:
-            print 'knot insertion failed: out of bounds!'
+            print('knot insertion failed: out of bounds!')
 
     def eval_basis(self, t=None, return_val=False):
         """
@@ -348,8 +352,7 @@ class BSpline(Struct):
             If True, label control points.
         """
         if self.curve_coors is None:
-            print 'curve coordinates not evaluated!'
-            raise ValueError
+            self.eval()
 
         cc = self.curve_coors
         cp = self.cp_coors
