@@ -3,19 +3,19 @@ from __future__ import absolute_import
 import numpy as nm
 import scipy.sparse as sps
 
-from sfepy.base.base import output, get_default, try_imports, Struct
+from sfepy.base.base import get_default, try_imports, Struct
 from sfepy.base.timing import Timer
 from sfepy.solvers.solvers import Solver, EigenvalueSolver
 import six
 from six.moves import range
 
 def eig(mtx_a, mtx_b=None, n_eigs=None, eigenvectors=True,
-        return_time=None, method='eig.scipy', **ckwargs):
+        return_time=None, solver_kind='eig.scipy', **ckwargs):
     """
     Utility function that constructs an eigenvalue solver given by
-    `method`, calls it and returns solution.
+    `solver_kind`, calls it and returns its output.
     """
-    kwargs = {'name' : 'aux', 'kind' : method}
+    kwargs = {'name' : 'aux', 'kind' : solver_kind}
     kwargs.update(ckwargs)
     conf = Struct(**kwargs)
     solver = Solver.any_from_conf(conf)
@@ -76,6 +76,10 @@ class ScipyEigenvalueSolver(EigenvalueSolver):
             see :func:`scipy.sparse.linalg.eigs()`
             or :func:`scipy.sparse.linalg.eigsh()`. For dense problmes,
             only 'LM' and 'SM' can be used"""),
+        ('linear_solver', "({'ls.cholesky', 'ls.mumps', ...}, ls_conf)",
+         None, False,
+         """The method used to construct an inverse linear operator. If None, the
+            eigenvalue solver will solve the linear system internally."""),
         ('*', '*', None, False,
          'Additional parameters supported by the method.'),
     ]
@@ -106,8 +110,36 @@ class ScipyEigenvalueSolver(EigenvalueSolver):
 
         else:
             eig = self.ssla.eigs if conf.method == 'eigs' else self.ssla.eigsh
-            out = eig(mtx_a, M=mtx_b, k=n_eigs, which=conf.which,
-                      return_eigenvectors=eigenvectors, **kwargs)
+            if conf.linear_solver is not None:
+                import sfepy.solvers.ls as ls
+                from sfepy.solvers import solver_table
+                import scipy.sparse as sps
+                from sfepy.base.base import Struct
+
+                ls_solvers = {}
+                for k, v in solver_table.items():
+                    if not k.startswith('ls'):
+                        continue
+
+                    aux = [par[0] == 'use_presolve' for par in v._parameters]
+                    if nm.any(aux):
+                        ls_solvers[k] = v
+
+                fake_mtx_a = sps.csc_matrix(mtx_a.shape)
+                solver_name, solver_conf = conf.linear_solver
+                ls_conf = Struct(use_presolve=True, **solver_conf)
+                ls = ls_solvers[solver_name](ls_conf)
+                ls.mtx = mtx_a
+                ls.presolve(mtx_a)
+                matvec = ls.__call__
+
+                inv_op_a = self.ssla.LinearOperator(mtx_a.shape, matvec=matvec)
+                out = eig(fake_mtx_a, M=mtx_b, k=n_eigs, which=conf.which,
+                          OPinv=inv_op_a,
+                          return_eigenvectors=eigenvectors, **kwargs)
+            else:
+                out = eig(mtx_a, M=mtx_b, k=n_eigs, which=conf.which,
+                          return_eigenvectors=eigenvectors, **kwargs)
 
         if eigenvectors:
             eigs = out[0]
@@ -222,7 +254,7 @@ class LOBPCGEigenvalueSolver(EigenvalueSolver):
     def __init__(self, conf, **kwargs):
         EigenvalueSolver.__init__(self, conf, **kwargs)
 
-        from scipy.sparse.linalg.eigen import lobpcg
+        from scipy.sparse.linalg import lobpcg
         self.lobpcg = lobpcg
 
     @standard_call
@@ -234,8 +266,8 @@ class LOBPCGEigenvalueSolver(EigenvalueSolver):
         else:
             n_eigs = min(n_eigs, mtx_a.shape[0])
 
-        x = nm.zeros((mtx_a.shape[0], n_eigs), dtype=nm.float64)
-        x[:n_eigs] = nm.eye(n_eigs, dtype=nm.float64)
+        rng = nm.random.default_rng(12345)
+        x = rng.normal(size=(mtx_a.shape[0], n_eigs))
 
         out = self.lobpcg(mtx_a, x, mtx_b,
                           M=conf.precond,
@@ -402,6 +434,17 @@ class MatlabEigenvalueSolver(EigenvalueSolver):
         EigenvalueSolver.__init__(self, conf, me=me, context=context,
                                   **kwargs)
 
+    def solver_call(self, mtx_filename, eigs_filename, which=None):
+        import os
+        import scipy.io as sio
+
+        eng = self.me.start_matlab()
+        eng.cd(os.path.dirname(__file__))
+        eng.matlab_eig(mtx_filename, eigs_filename)
+        eng.quit()
+
+        return sio.loadmat(eigs_filename)
+
     @standard_call
     def __call__(self, mtx_a, mtx_b=None, n_eigs=None, eigenvectors=None,
                  status=None, conf=None, comm=None, context=None):
@@ -419,7 +462,8 @@ class MatlabEigenvalueSolver(EigenvalueSolver):
             'A' : mtx_a,
             'B' : mtx_b if mtx_b is not None else 'None',
             'n_eigs' : n_eigs if n_eigs is not None else 'None',
-            'eigenvectors' : eigenvectors,
+            'eigenvectors' : (eigenvectors
+                              if eigenvectors is not None else False),
             'method' : conf.method,
             'balance' : conf.balance,
             'algorithm' : conf.algorithm,
@@ -428,12 +472,7 @@ class MatlabEigenvalueSolver(EigenvalueSolver):
             'eigs_options' : solver_kwargs,
         })
 
-        eng = self.me.start_matlab()
-        eng.cd(os.path.dirname(__file__))
-        eng.matlab_eig(mtx_filename, eigs_filename)
-        eng.quit()
-
-        evp = sio.loadmat(eigs_filename)
+        evp = self.solver_call(mtx_filename, eigs_filename, conf.which)
 
         shutil.rmtree(dirname)
 
@@ -442,3 +481,102 @@ class MatlabEigenvalueSolver(EigenvalueSolver):
             out =  (out, evp['vecs'])
 
         return out
+
+
+class OctaveEigenvalueSolver(MatlabEigenvalueSolver):
+    """
+    Octave eigenvalue problem solver.
+    """
+    name = 'eig.octave'
+
+    def __init__(self, conf, comm=None, context=None, **kwargs):
+        from oct2py import octave as oe
+
+        EigenvalueSolver.__init__(self, conf, oe=oe, context=context,
+                                  **kwargs)
+
+    def solver_call(self, mtx_filename, eigs_filename, which):
+        import os
+        import scipy.io as sio
+
+        self.oe.addpath(os.path.dirname(__file__))
+        self.oe.matlab_eig(mtx_filename, eigs_filename)
+        self.oe.exit()
+
+        evp = sio.loadmat(eigs_filename)
+        evals = evp['vals']
+
+        which = which.lower()
+        sort_funs = {
+            'lm': (nm.abs, -1),
+            'sm': (nm.abs, 1),
+            'la': (nm.real, -1),
+            'sa': (nm.real, 1),
+            'lr': (nm.real, -1),
+            'sr': (nm.real, 1),
+            'li': (nm.imag, -1),
+            'si': (nm.imag, 1),
+            'be': (nm.real, 1),
+        }
+
+        sfun, order = sort_funs[which]
+        idxs = nm.argsort(sfun(evals), axis=0)
+        idxs = idxs.ravel()[::order]
+        if which == 'be':
+            idxs = nm.hstack([idxs[::-1], idxs])
+            k2 = len(idxs) // 2
+            idxs = idxs[k2:(k2 + len(idxs))]
+
+        out = {'vals': evals[idxs, :]}
+        if 'vecs' in evp:
+            out['vecs'] = evp['vecs'][:, idxs]
+
+        return out
+
+
+class PrimmeEigenvalueSolver(EigenvalueSolver):
+    """
+    PRIMME eigenvalue problem solver.
+
+    https://github.com/primme/primme
+
+    Installation: pip install primme
+    """
+    name = 'eig.primme'
+
+    _parameters = [
+        ('which',
+         "{'LM', 'SM', 'LA', 'SA', 'CLT', 'CGT'}", 'LM', False,
+         'Which eigenvectors and eigenvalues to find.'),
+        ('sigma', 'float', None, False,
+         'Find eigenvalues near sigma.'),
+        ('maxiter', 'int', None, False, 'Maximum number of iterations.'),
+        ('tol', 'float', 0, False,
+         'Tolerance for eigenpairs (stopping criterion).'),
+        ('*', '*', None, False,
+         'Additional parameters supported by eigsh().'),
+    ]
+
+    def __init__(self, conf, comm=None, context=None, **kwargs):
+        import primme
+
+        EigenvalueSolver.__init__(self, conf, primme=primme, context=context,
+                                  **kwargs)
+
+    @standard_call
+    def __call__(self, mtx_a, mtx_b=None, n_eigs=None, eigenvectors=None,
+                 status=None, conf=None, comm=None, context=None):
+        if n_eigs is None:
+            n_eigs = mtx_a.shape[0]
+
+        solver_kwargs = self.build_solver_kwargs(conf)
+
+        out = self.primme.eigsh(mtx_a, n_eigs, M=mtx_b,
+                                which=conf.which.upper(),
+                                tol=conf.tol,
+                                maxiter=conf.maxiter,
+                                sigma=conf.sigma,
+                                return_eigenvectors=eigenvectors,
+                                **solver_kwargs)
+
+        return out
