@@ -1,10 +1,8 @@
 """
 Base (abstract) solver classes.
 """
-from __future__ import absolute_import
 import numpy as nm
 from sfepy.base.base import Struct, output, structify
-import six
 
 def make_get_conf(conf, kwargs):
     def _get_conf_item(name, default=None, msg_if_none=None):
@@ -62,7 +60,7 @@ def make_option_docstring(name, kind, default, required, doc):
     return entry
 
 
-def use_first_available(solver_list, context=None):
+def use_first_available(solver_list, context=None, **kwargs):
     """
     Use the first available solver from `solver_list`.
 
@@ -72,6 +70,8 @@ def use_first_available(solver_list, context=None):
         The list of solver names or configuration objects.
     context : object, optional
         An optional solver context to pass to the solver.
+    **kwargs : keyword arguments
+        Additional solver options, see the particular __init__() methods.
 
     Returns
     -------
@@ -88,9 +88,9 @@ def use_first_available(solver_list, context=None):
             _conf = conf.copy()
             _conf.__dict__.pop('fallback', None)
             if isinstance(solver, str):
-                out = Solver.any_from_conf(_conf, context=context)
+                out = Solver.any_from_conf(_conf, context=context, **kwargs)
             else:
-                out = solver(_conf, contex=context)
+                out = solver(_conf, contex=context, **kwargs)
 
             output("using '%s' solver" % name)
             break
@@ -143,8 +143,7 @@ class SolverMeta(type):
 
         return super(SolverMeta, cls).__new__(cls, name, bases, ndict)
 
-@six.add_metaclass(SolverMeta)
-class Solver(Struct):
+class Solver(Struct, metaclass=SolverMeta):
     """
     Base class for all solver kinds. Takes care of processing of common
     configuration options.
@@ -166,6 +165,8 @@ class Solver(Struct):
         ('kind', 'str', None, True,
          """The solver kind, as given by the `name` class attribute of the
             Solver subclasses."""),
+        ('report_status', 'bool', False, False,
+         'If True, the solver can report the solution status.'),
         ('verbose', 'bool', False, False,
          """If True, the solver can print more information about the
             solution."""),
@@ -209,6 +210,9 @@ class Solver(Struct):
         elif isinstance(conf, dict):
             conf = Struct(**conf)
 
+        else:
+            conf = conf.copy()
+
         if conf.get('name', None) is None:
             conf.name = 'auto_' + self.__class__.__name__
 
@@ -238,7 +242,7 @@ class Solver(Struct):
         std = set([ii[0] for ii in options if ii[0] != '*'])
 
         kwargs = {}
-        for key, val in six.iteritems(conf.to_dict()):
+        for key, val in conf.to_dict().items():
             if key not in std:
                 kwargs[key] = val
 
@@ -271,6 +275,9 @@ class LinearSolver(Solver):
         """
         return self.conf.get('eps_a', None), self.conf.get('eps_r', None)
 
+    def clear(self):
+        pass
+
     def presolve(self, mtx):
         pass
 
@@ -289,16 +296,34 @@ class NonlinearSolver(Solver):
                  lin_solver=None, iter_hook=None, status=None):
         raise ValueError('called an abstract NonlinearSolver instance!')
 
+class TimeStepController(Solver):
+    """
+    Abstract time step controller class.
+    """
+
+    def __init__(self, conf, **kwargs):
+        Solver.__init__(self, conf=conf, **kwargs)
+
+    def get_initial_dt(self, ts, vec, **kwargs):
+        return ts.dt
+
+    def __call__(self, ts, vec0, vec1, **kwargs):
+        raise ValueError('called an abstract TimeStepController instance!')
+
 class TimeSteppingSolver(Solver):
     """
     Abstract time stepping solver class.
     """
 
-    def __init__(self, conf, nls=None, status=None, context=None, **kwargs):
-        Solver.__init__(self, conf=conf, nls=nls, status=status,
+    def __init__(self, conf, nls=None, status=None, tsc=None, context=None,
+                 **kwargs):
+        Solver.__init__(self, conf=conf, nls=nls, tsc=tsc, status=status,
                         context=context, **kwargs)
 
-    def __call__(self, vec0=None, nls=None, init_fun=None,
+    def set_dof_info(self, di):
+        self.di = di
+
+    def __call__(self, vec0=None, nls=None, tsc=None, init_fun=None,
                  prestep_fun=None, poststep_fun=None,
                  status=None, **kwargs):
         raise ValueError('called an abstract TimeSteppingSolver instance!')
@@ -358,7 +383,7 @@ class QuadraticEVPSolver(Solver):
         Solver.__init__(self, conf=conf, mtx_m=mtx_m, mtx_d=mtx_d, mtx_k=mtx_k,
                         n_eigs=n_eigs, eigenvectors=eigenvectors,
                         status=status, context=context)
-        solver_conf = structify(conf.solver)
+        solver_conf = structify(self.conf.solver)
         self.solver = Solver.any_from_conf(solver_conf)
 
     def __call__(self, mtx_m, mtx_d, mtx_k, n_eigs=None,
