@@ -1,8 +1,9 @@
+from __future__ import absolute_import
 import sfepy
-import terms
-import extmods
-from terms import Terms, Term, CharacteristicFunction, vector_chunk_generator
-from terms_th import THTerm, ETHTerm
+from . import terms
+from . import extmods
+from .terms import Terms, Term
+from .terms_th import THTerm, ETHTerm
 from sfepy.base.base import load_classes
 
 term_files = sfepy.get_paths('sfepy/terms/terms*.py')
