@@ -1,49 +1,61 @@
 import numpy as nm
 
-from sfepy.base.base import assert_
 from sfepy.linalg import dot_sequences
 from sfepy.terms.terms import Term, terms
 from sfepy.terms.terms_th import THTerm, ETHTerm
 
-class DotProductVolumeTerm(Term):
+
+class DotProductTerm(Term):
     r"""
-    Volume :math:`L^2(\Omega)` weighted dot product for both scalar and vector
-    fields. Can be evaluated. Can use derivatives.
+    Volume and surface :math:`L^2()` weighted dot product for both
+    scalar and vector fields. If the region is a surface and
+    either virtual or state variable is a vector, the orientation
+    of the normal vectors is outwards to the parent region of the virtual
+    variable. Can be evaluated. Can use derivatives.
 
     :Definition:
 
     .. math::
-        \int_\Omega q p \mbox{ , } \int_\Omega \ul{v} \cdot \ul{u}
-        \mbox{ , }
-        \int_\Omega p r \mbox{ , } \int_\Omega \ul{u} \cdot \ul{w} \\
-        \int_\Omega c q p \mbox{ , } \int_\Omega c \ul{v} \cdot \ul{u}
-        \mbox{ , }
-        \int_\Omega c p r \mbox{ , } \int_\Omega c \ul{u} \cdot \ul{w} \\
-        \int_\Omega \ul{v} \cdot (\ull{M} \ul{u})
-        \mbox{ , }
-        \int_\Omega \ul{u} \cdot (\ull{M} \ul{w})
+        \int_{\cal{D}} q p \mbox{ , }
+        \int_{\cal{D}} \ul{v} \cdot \ul{u}\\
+        \int_\Gamma \ul{v} \cdot \ul{n} p \mbox{ , }
+        \int_\Gamma q \ul{n} \cdot \ul{u} \mbox{ , }\\
+        \int_{\cal{D}} c q p \mbox{ , }
+        \int_{\cal{D}} c \ul{v} \cdot \ul{u} \mbox{ , }
+        \int_{\cal{D}} \ul{v} \cdot \ull{c} \cdot \ul{u}
 
-    :Arguments 1:
-        - material : :math:`c` or :math:`\ull{M}` (optional)
-        - virtual  : :math:`q` or :math:`\ul{v}`
-        - state    : :math:`p` or :math:`\ul{u}`
-
-    :Arguments 2:
-        - material    : :math:`c` or :math:`\ull{M}` (optional)
-        - parameter_1 : :math:`p` or :math:`\ul{u}`
-        - parameter_2 : :math:`r` or :math:`\ul{w}`
+    :Arguments:
+        - material: :math:`c` or :math:`\ull{c}` (optional)
+        - virtual/parameter_1: :math:`q` or :math:`\ul{v}`
+        - state/parameter_2: :math:`p` or :math:`\ul{u}`
     """
-    name = 'dw_volume_dot'
+    name = 'dw_dot'
     arg_types = (('opt_material', 'virtual', 'state'),
                  ('opt_material', 'parameter_1', 'parameter_2'))
-    arg_shapes = [{'opt_material' : '1, 1', 'virtual' : (1, 'state'),
+    arg_shapes_dict = {
+        'cell': [{'opt_material' : '1, 1', 'virtual' : (1, 'state'),
+                  'state' : 1, 'parameter_1' : 1, 'parameter_2' : 1},
+                 {'opt_material' : None},
+                 {'opt_material' : '1, 1', 'virtual' : ('D', 'state'),
+                  'state' : 'D', 'parameter_1' : 'D', 'parameter_2' : 'D'},
+                 {'opt_material' : 'D, D'},
+                 {'opt_material' : None}],
+        'facet': [{'opt_material' : '1, 1', 'virtual' : (1, 'state'),
                    'state' : 1, 'parameter_1' : 1, 'parameter_2' : 1},
                   {'opt_material' : None},
+                  {'opt_material' : '1, 1', 'virtual' : (1, None),
+                   'state' : 'D'},
+                  {'opt_material' : None},
+                  {'opt_material' : '1, 1', 'virtual' : ('D', None),
+                   'state' : 1},
+                  {'opt_material' : None},
                   {'opt_material' : '1, 1', 'virtual' : ('D', 'state'),
                    'state' : 'D', 'parameter_1' : 'D', 'parameter_2' : 'D'},
                   {'opt_material' : 'D, D'},
                   {'opt_material' : None}]
+    }
     modes = ('weak', 'eval')
+    integration = ('cell', 'facet')
 
     @staticmethod
     def dw_dot(out, mat, val_qp, vgeo, sgeo, fun, fmode):
@@ -86,7 +98,7 @@ class DotProductVolumeTerm(Term):
         if mode == 'weak':
             if mat is None:
                 n_cell, n_qp, dim, n_n, n_c = self.get_data_shape(state)
-                mat = nm.ones((n_cell, n_qp, 1, 1), dtype=nm.float64)
+                mat = nm.ones((1, n_qp, 1, 1), dtype=nm.float64)
 
             sgeo, _ = self.get_mapping(state)
 
@@ -99,7 +111,7 @@ class DotProductVolumeTerm(Term):
                 fmode = 1
 
             if state.n_components > 1:
-                if ((self.integration == 'volume')
+                if ((self.act_integration == 'cell')
                     or (virtual.n_components > 1)):
                     fun = terms.dw_volume_dot_vector
 
@@ -107,7 +119,7 @@ class DotProductVolumeTerm(Term):
                     fun = terms.dw_surface_s_v_dot_n
 
             else:
-                if ((self.integration == 'volume')
+                if ((self.act_integration == 'cell')
                     or (virtual.n_components == 1)):
                     fun = terms.dw_volume_dot_scalar
 
@@ -139,59 +151,8 @@ class DotProductVolumeTerm(Term):
         else:
             self.function = self.d_dot
 
-class DotProductSurfaceTerm(DotProductVolumeTerm):
-    r"""
-    Surface :math:`L^2(\Gamma)` dot product for both scalar and vector
-    fields. If either virtual or state variable is a vector, the orientation
-    of the normal vectors is outwards to the parent region of the virtual
-    variable.
 
-    :Definition:
-
-    .. math::
-        \int_\Gamma q p \mbox{ , } \int_\Gamma \ul{v} \cdot \ul{u}
-        \mbox{ , }
-        \int_\Gamma \ul{v} \cdot \ul{n} p \mbox{ , }
-        \int_\Gamma q \ul{n} \cdot \ul{u} \mbox{ , }
-        \int_\Gamma p r \mbox{ , } \int_\Gamma \ul{u} \cdot \ul{w}
-        \mbox{ , } \int_\Gamma \ul{w} \cdot \ul{n} p \\
-        \int_\Gamma c q p \mbox{ , } \int_\Gamma c \ul{v} \cdot \ul{u}
-        \mbox{ , }
-        \int_\Gamma c p r \mbox{ , } \int_\Gamma c \ul{u} \cdot \ul{w} \\
-        \int_\Gamma \ul{v} \cdot \ull{M} \cdot \ul{u}
-        \mbox{ , }
-        \int_\Gamma \ul{u} \cdot \ull{M} \cdot \ul{w}
-
-    :Arguments 1:
-        - material : :math:`c` or :math:`\ull{M}` (optional)
-        - virtual  : :math:`q` or :math:`\ul{v}`
-        - state    : :math:`p` or :math:`\ul{u}`
-
-    :Arguments 2:
-        - material    : :math:`c` or :math:`\ull{M}` (optional)
-        - parameter_1 : :math:`p` or :math:`\ul{u}`
-        - parameter_2 : :math:`r` or :math:`\ul{w}`
-    """
-    name = 'dw_surface_dot'
-    arg_types = (('opt_material', 'virtual', 'state'),
-                 ('opt_material', 'parameter_1', 'parameter_2'))
-    arg_shapes = [{'opt_material' : '1, 1', 'virtual' : (1, 'state'),
-                   'state' : 1, 'parameter_1' : 1, 'parameter_2' : 1},
-                  {'opt_material' : None},
-                  {'opt_material' : '1, 1', 'virtual' : (1, None),
-                   'state' : 'D'},
-                  {'opt_material' : None},
-                  {'opt_material' : '1, 1', 'virtual' : ('D', None),
-                   'state' : 1},
-                  {'opt_material' : None},
-                  {'opt_material' : '1, 1', 'virtual' : ('D', 'state'),
-                   'state' : 'D', 'parameter_1' : 'D', 'parameter_2' : 'D'},
-                  {'opt_material' : 'D, D'},
-                  {'opt_material' : None}]
-    modes = ('weak', 'eval')
-    integration = 'surface'
-
-class BCNewtonTerm(DotProductSurfaceTerm):
+class BCNewtonTerm(DotProductTerm):
     r"""
     Newton boundary condition term.
 
@@ -211,12 +172,14 @@ class BCNewtonTerm(DotProductSurfaceTerm):
     arg_shapes = {'material_1' : '1, 1', 'material_2' : '1, 1',
                   'virtual' : (1, 'state'), 'state' : 1}
     mode = 'weak'
+    integration = 'facet'
+    arg_shapes_dict = None
 
     def get_fargs(self, alpha, p_outer, virtual, state,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
-        fargs = DotProductSurfaceTerm.get_fargs(self, alpha, virtual, state,
-                                                mode, term_mode, diff_var,
-                                                **kwargs)
+        fargs = DotProductTerm.get_fargs(self, alpha, virtual, state,
+                                         mode, term_mode, diff_var,
+                                         **kwargs)
         fargs = fargs[:1] + (fargs[1] - p_outer,) + fargs[2:]
 
         return fargs
@@ -322,23 +285,18 @@ class VectorDotGradScalarTerm(Term):
         \int_{\Omega} \ul{u} \cdot \nabla q \\
         \int_{\Omega} c \ul{v} \cdot \nabla p \mbox{ , }
         \int_{\Omega} c \ul{u} \cdot \nabla q \\
-        \int_{\Omega} \ul{v} \cdot (\ull{M} \nabla p) \mbox{ , }
-        \int_{\Omega} \ul{u} \cdot (\ull{M} \nabla q)
+        \int_{\Omega} \ul{v} \cdot (\ull{c} \nabla p) \mbox{ , }
+        \int_{\Omega} \ul{u} \cdot (\ull{c} \nabla q)
 
     :Arguments 1:
-        - material : :math:`c` or :math:`\ull{M}` (optional)
-        - virtual  : :math:`\ul{v}`
-        - state    : :math:`p`
+        - material: :math:`c` or :math:`\ull{c}` (optional)
+        - virtual/parameter_v: :math:`\ul{v}`
+        - state/parameter_s: :math:`p`
 
     :Arguments 2:
-        - material : :math:`c` or :math:`\ull{M}` (optional)
+        - material : :math:`c` or :math:`\ull{c}` (optional)
         - state    : :math:`\ul{u}`
         - virtual  : :math:`q`
-
-    :Arguments 3:
-        - material    : :math:`c` or :math:`\ull{M}` (optional)
-        - parameter_v : :math:`\ul{u}`
-        - parameter_s : :math:`p`
     """
     name = 'dw_v_dot_grad_s'
     arg_types = (('opt_material', 'virtual', 'state'),
@@ -400,7 +358,7 @@ class VectorDotGradScalarTerm(Term):
         self.function = {
             'v_weak' : terms.dw_v_dot_grad_s_vw,
             's_weak' : terms.dw_v_dot_grad_s_sw,
-            'eval' : DotProductVolumeTerm.d_dot,
+            'eval' : DotProductTerm.d_dot,
         }[self.mode]
 
 class VectorDotScalarTerm(Term):
@@ -411,23 +369,18 @@ class VectorDotScalarTerm(Term):
     :Definition:
 
     .. math::
-        \int_{\Omega} \ul{v} \cdot \ul{m} p \mbox{ , }
-        \int_{\Omega} \ul{u} \cdot \ul{m} q\\
+        \int_{\Omega} \ul{v} \cdot \ul{c} p \mbox{ , }
+        \int_{\Omega} \ul{u} \cdot \ul{c} q\\
 
     :Arguments 1:
-        - material : :math:`\ul{m}`
-        - virtual  : :math:`\ul{v}`
-        - state    : :math:`p`
+        - material : :math:`\ul{c}`
+        - virtual/parameter_v: :math:`\ul{v}`
+        - state/parameter_s: :math:`p`
 
     :Arguments 2:
-        - material : :math:`\ul{m}`
+        - material : :math:`\ul{c}`
         - state    : :math:`\ul{u}`
         - virtual  : :math:`q`
-
-    :Arguments 3:
-        - material    : :math:`\ul{m}`
-        - parameter_v : :math:`\ul{u}`
-        - parameter_s : :math:`p`
     """
     name = 'dw_vm_dot_s'
     arg_types = (('material', 'virtual', 'state'),
@@ -451,10 +404,10 @@ class VectorDotScalarTerm(Term):
             status1 = terms.actBfT(aux, bfve, mat)
 
         if fmode == 0:
-            status2 = terms.mulAB_integrate(out, aux, val_qp, geo, 'AB')
+            status2 = terms.mulAB_integrate(out, aux, val_qp, geo.cmap, 'AB')
 
         if fmode == 1:
-            status2 = terms.mulAB_integrate(out, aux, bfsc, geo, 'AB')
+            status2 = terms.mulAB_integrate(out, aux, bfsc, geo.cmap, 'AB')
 
         if fmode == 2:
             aux = (bfsc * dot_sequences(mat, val_qp,
@@ -462,7 +415,7 @@ class VectorDotScalarTerm(Term):
             status2 = geo.integrate(out, nm.ascontiguousarray(aux))
 
         if fmode == 3:
-            status2 = terms.mulAB_integrate(out, bfsc, aux, geo, 'ATBT')
+            status2 = terms.mulAB_integrate(out, bfsc, aux, geo.cmap, 'ATBT')
 
         return status1 and status2
 
@@ -494,7 +447,6 @@ class VectorDotScalarTerm(Term):
 
             else:
                 qp_var, geo, fmode = vvar, vgs, 2
-                bfve, bfsc = bfsc, bfve
 
             if diff_var is None:
                 val_qp = self.get(qp_var, 'val')
@@ -556,13 +508,13 @@ class ScalarDotGradIScalarTerm(Term):
 
         if fmode == 0:
             status = terms.mulAB_integrate(out, bft,
-                                           cc(grad[..., idx:idx+1, :]), vg,
-                                           mode='ATB')
+                                           cc(grad[..., idx:idx+1, :]),
+                                           vg.cmap, mode='ATB')
 
         else:
             status = terms.mulAB_integrate(out, bft,
-                                           cc(vg.bfg[:,:,idx:(idx + 1),:]), vg,
-                                           mode='ATB')
+                                           cc(vg.bfg[:,:,idx:(idx + 1),:]),
+                                           vg.cmap, mode='ATB')
 
         return status
 
@@ -580,7 +532,7 @@ class ScalarDotGradIScalarTerm(Term):
             vg, _ = self.get_mapping(virtual)
             vgs, _ = self.get_mapping(state)
 
-            idx = int(material)
+            idx = int(material[0, 0])
 
             return vgs.bf, vg, grad, idx, fmode
 
