@@ -4,6 +4,7 @@ from sfepy.linalg import dot_sequences
 from sfepy.homogenization.utils import iter_sym
 from sfepy.terms.terms import Term, terms
 from sfepy.terms.terms_th import THTerm, ETHTerm
+import sfepy.mechanics.membranes as membranes
 
 ## expr = """
 ## e = 1/2 * (grad( vec( u ) ) + grad( vec( u ) ).T)
@@ -96,20 +97,15 @@ class LinearElasticIsotropicTerm(LinearElasticTerm):
     :Definition:
 
     .. math::
-        \int_{\Omega} D_{ijkl}\ e_{ij}(\ul{v}) e_{kl}(\ul{u}) \mbox{ with }
+        \int_{\Omega} D_{ijkl}\ e_{ij}(\ul{v}) e_{kl}(\ul{u})\\ \mbox{ with } \\
         D_{ijkl} = \mu (\delta_{ik} \delta_{jl}+\delta_{il} \delta_{jk}) +
         \lambda \ \delta_{ij} \delta_{kl}
 
     :Arguments:
-        - material_1 : :math:`\lambda`
-        - material_2 : :math:`\mu`
-        - virtual    : :math:`\ul{v}`
-        - state      : :math:`\ul{u}`
-
-    :Arguments 2:
-        - material    : :math:`D_{ijkl}`
-        - parameter_1 : :math:`\ul{w}`
-        - parameter_2 : :math:`\ul{u}`
+        - material_1: :math:`\lambda`
+        - material_2: :math:`\mu`
+        - virtual/parameter_1: :math:`\ul{v}`
+        - state/parameter_2: :math:`\ul{u}`
     """
     name = 'dw_lin_elastic_iso'
     arg_types = (('material_1', 'material_2', 'virtual', 'state'),
@@ -150,16 +146,16 @@ class SDLinearElasticTerm(Term):
         - material    : :math:`D_{ijkl}`
         - parameter_w : :math:`\ul{w}`
         - parameter_u : :math:`\ul{u}`
-        - parameter_mesh_velocity : :math:`\ul{\Vcal}`
+        - parameter_mv : :math:`\ul{\Vcal}`
     """
-    name = 'd_sd_lin_elastic'
+    name = 'ev_sd_lin_elastic'
     arg_types = ('material', 'parameter_w', 'parameter_u',
-                 'parameter_mesh_velocity')
+                 'parameter_mv')
     arg_shapes = {'material' : 'S, S',
                   'parameter_w' : 'D', 'parameter_u' : 'D',
-                  'parameter_mesh_velocity' : 'D'}
+                  'parameter_mv' : 'D'}
     geometries = ['2_3', '2_4', '3_4', '3_8']
-    function = terms.d_sd_lin_elastic
+    function = staticmethod(terms.d_sd_lin_elastic)
 
     def get_fargs(self, mat, par_w, par_u, par_mv,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
@@ -400,14 +396,7 @@ class CauchyStrainTerm(Term):
     :Definition:
 
     .. math::
-        \int_{\Omega} \ull{e}(\ul{w})
-
-    .. math::
-        \mbox{vector for } K \from \Ical_h: \int_{T_K} \ull{e}(\ul{w}) /
-        \int_{T_K} 1
-
-    .. math::
-        \ull{e}(\ul{w})|_{qp}
+        \int_{\cal{D}} \ull{e}(\ul{w})
 
     :Arguments:
         - parameter : :math:`\ul{w}`
@@ -415,6 +404,7 @@ class CauchyStrainTerm(Term):
     name = 'ev_cauchy_strain'
     arg_types = ('parameter',)
     arg_shapes = {'parameter' : 'D'}
+    integration = ('cell', 'facet_extra')
 
     @staticmethod
     def function(out, strain, vg, fmode):
@@ -423,7 +413,7 @@ class CauchyStrainTerm(Term):
             status = 0
 
         else:
-            status = terms.de_cauchy_strain(out, strain, vg, fmode)
+            status = terms.de_cauchy_strain(out, strain, vg.cmap, fmode)
 
         return status
 
@@ -446,32 +436,6 @@ class CauchyStrainTerm(Term):
 
         return (n_el, n_qp, dim * (dim + 1) // 2, 1), parameter.dtype
 
-class CauchyStrainSTerm(CauchyStrainTerm):
-    r"""
-    Evaluate Cauchy strain tensor on a surface region.
-
-    See :class:`CauchyStrainTerm`.
-
-    Supports 'eval', 'el_avg' and 'qp' evaluation modes.
-
-    :Definition:
-
-    .. math::
-        \int_{\Gamma} \ull{e}(\ul{w})
-
-    .. math::
-        \mbox{vector for } K \from \Ical_h: \int_{T_K} \ull{e}(\ul{w}) /
-        \int_{T_K} 1
-
-    .. math::
-        \ull{e}(\ul{w})|_{qp}
-
-    :Arguments:
-        - parameter : :math:`\ul{w}`
-    """
-    name = 'ev_cauchy_strain_s'
-    arg_types = ('parameter',)
-    integration = 'surface_extra'
 
 class CauchyStressTerm(Term):
     r"""
@@ -486,14 +450,7 @@ class CauchyStressTerm(Term):
     :Definition:
 
     .. math::
-        \int_{\Omega} D_{ijkl} e_{kl}(\ul{w})
-
-    .. math::
-        \mbox{vector for } K \from \Ical_h:
-        \int_{T_K} D_{ijkl} e_{kl}(\ul{w}) / \int_{T_K} 1
-
-    .. math::
-        D_{ijkl} e_{kl}(\ul{w})|_{qp}
+        \int_{\cal{D}} D_{ijkl} e_{kl}(\ul{w})
 
     :Arguments:
         - material  : :math:`D_{ijkl}`
@@ -502,6 +459,7 @@ class CauchyStressTerm(Term):
     name = 'ev_cauchy_stress'
     arg_types = ('material', 'parameter')
     arg_shapes = {'material' : 'S, S', 'parameter' : 'D'}
+    integration = ('cell', 'facet_extra')
 
     @staticmethod
     def function(out, coef, strain, mat, vg, fmode):
@@ -510,7 +468,7 @@ class CauchyStressTerm(Term):
             status = 0
 
         else:
-            status = terms.de_cauchy_stress(out, strain, mat, vg, fmode)
+            status = terms.de_cauchy_stress(out, strain, mat, vg.cmap, fmode)
 
         if coef is not None:
             out *= coef
@@ -552,14 +510,6 @@ class CauchyStressTHTerm(CauchyStressTerm, THTerm):
         \int_{\Omega} \int_0^t \Hcal_{ijkl}(t-\tau)\,e_{kl}(\ul{w}(\tau))
         \difd{\tau}
 
-    .. math::
-        \mbox{vector for } K \from \Ical_h:
-        \int_{T_K} \int_0^t \Hcal_{ijkl}(t-\tau)\,e_{kl}(\ul{w}(\tau))
-        \difd{\tau} / \int_{T_K} 1
-
-    .. math::
-        \int_0^t \Hcal_{ijkl}(t-\tau)\,e_{kl}(\ul{w}(\tau)) \difd{\tau}|_{qp}
-
     :Arguments:
         - ts        : :class:`TimeStepper` instance
         - material  : :math:`\Hcal_{ijkl}(\tau)`
@@ -610,14 +560,6 @@ class CauchyStressETHTerm(CauchyStressTerm, ETHTerm):
         \int_{\Omega} \int_0^t \Hcal_{ijkl}(t-\tau)\,e_{kl}(\ul{w}(\tau))
         \difd{\tau}
 
-    .. math::
-        \mbox{vector for } K \from \Ical_h:
-        \int_{T_K} \int_0^t \Hcal_{ijkl}(t-\tau)\,e_{kl}(\ul{w}(\tau))
-        \difd{\tau} / \int_{T_K} 1
-
-    .. math::
-        \int_0^t \Hcal_{ijkl}(t-\tau)\,e_{kl}(\ul{w}(\tau)) \difd{\tau}|_{qp}
-
     :Arguments:
         - ts         : :class:`TimeStepper` instance
         - material_0 : :math:`\Hcal_{ijkl}(0)`
@@ -807,10 +749,11 @@ class ElasticWaveTerm(Term):
 
         ebf = expand_basis(geo.bf, dim)
 
+        mat = Term.tile_mat(mat, n_el)
         gmat = _build_wave_strain_op(kappa, ebf)
 
         if diff_var is None:
-            econn = state.field.get_econn('volume', self.region)
+            econn = state.field.get_econn('cell', self.region)
             adc = create_adof_conn(nm.arange(state.n_dof, dtype=nm.int32),
                                    econn, n_c, 0)
             vals = state()[adc]
@@ -838,7 +781,7 @@ class ElasticWaveCauchyTerm(Term):
     :Definition:
 
     .. math::
-        \int_{\Omega} D_{ijkl}\ g_{ij}(\ul{v}) e_{kl}(\ul{u}) \;,
+        \int_{\Omega} D_{ijkl}\ g_{ij}(\ul{v}) e_{kl}(\ul{u})\\
         \int_{\Omega} D_{ijkl}\ g_{ij}(\ul{u}) e_{kl}(\ul{v})
 
     :Arguments 1:
@@ -876,12 +819,13 @@ class ElasticWaveCauchyTerm(Term):
 
         ebf = expand_basis(geo.bf, dim)
 
+        mat = Term.tile_mat(mat, n_el)
         gmat = _build_wave_strain_op(kappa, ebf)
         emat = _build_cauchy_strain_op(geo.bfg)
 
         if diff_var is None:
             avar = evar if self.mode == 'ge' else gvar
-            econn = avar.field.get_econn('volume', self.region)
+            econn = avar.field.get_econn('cell', self.region)
             adc = create_adof_conn(nm.arange(avar.n_dof, dtype=nm.int32),
                                    econn, n_c, 0)
             vals = avar()[adc]
@@ -907,3 +851,346 @@ class ElasticWaveCauchyTerm(Term):
             fmode = 1
 
         return out_qp, geo, fmode
+
+
+class LinearSpringTerm(Term):
+    r"""Linear spring element.
+
+    :Definition:
+
+    .. math::
+        \ul{f}^{(i)} = - \ul{f}^{(j)} = k (\ul{u}^{(j)} - \ul{u}^{(i)})\\
+        \quad \forall \mbox{ elements } T_K^{i,j}\\
+        \mbox{ in a region connecting nodes } i, j
+
+    :Arguments 1:
+        - material : :math:`k`
+        - virtual  : :math:`\ul{v}`
+        - state    : :math:`\ul{u}`
+    """
+    name = 'dw_lin_spring'
+    arg_types = ('material', 'virtual', 'state')
+    arg_shapes = {'material': '1, 1', 'virtual': ('D', 'state'), 'state': 'D'}
+    integration_order = 0
+    geometries = ['1_2', '2_1_2', '3_1_2']
+
+    @staticmethod
+    def function(out, stiffness, vec, diff_var):
+        dim = out.shape[-2] // 2
+        if diff_var is None:
+            aux = nm.array([-1, 1])
+            for k in nm.arange(dim) * 2:
+                du = (vec[:, k] - vec[:, k + 1])[:, None]
+                out[:, 0, k:(k + 2), 0] = aux * du
+
+            out *= stiffness
+
+        else:
+            eye = nm.eye(2 * dim, 2 * dim, dtype=nm.float64)
+            eye.shape = (1, 1) + eye.shape
+            out[...] = - stiffness * eye
+            for k in nm.arange(dim) * 2:
+                out[..., k, k + 1] = out[..., k + 1, k] = 1
+
+        return 0
+
+    def get_fargs(self, mat, virtual, state,
+                  mode=None, term_mode=None, diff_var=None, **kwargs):
+
+        if diff_var is None:
+            from sfepy.discrete.variables import create_adof_conn
+            econn = virtual.field.get_econn('cell', self.region)
+            _, _, _, _, n_c = self.get_data_shape(virtual)
+            adc = create_adof_conn(nm.arange(state.n_dof, dtype=nm.int32),
+                                   econn, n_c, 0)
+
+            return mat, state()[adc], diff_var
+        else:
+            return mat, None, diff_var
+
+
+class LinearTrussTerm(Term):
+    r"""
+    Evaluate internal force in the element direction. To be used with
+    'el_avg' or 'qp' evaluation modes which give the same results.
+    The material parameter :math:`EA` is equal to
+    Young modulus times element coss-section. The internal force
+    is given by :math:`F^{(i)} = -F^{(j)} = EA / l (U^{(j)} - U^{(i)})`,
+    where :math:`l` is the element length and :math:`U`, :math:`F` are
+    the nodal displacements and the nodal forces in the element direction.
+
+    :Definition:
+
+    .. math::
+        F^{(i)} = -F^{(j)} = EA / l (U^{(j)} - U^{(i)})\\
+        \quad \forall \mbox{ elements } T_K^{i,j}\\
+        \mbox{ in a region connecting nodes } i, j
+
+    :Arguments:
+        - material : :math:`EA`
+        - parameter : :math:`\ul{w}`
+    """
+    name = 'dw_lin_truss'
+    arg_types = ('material', 'virtual', 'state')
+    arg_shapes = {'material': '1, 1', 'virtual': ('D', 'state'),
+                  'state': 'D'}
+    integration_order = 0
+    geometries = ['1_2', '2_1_2', '3_1_2']
+
+    @staticmethod
+    def function(out, mat, vec, mtx_t, length, diff_var):
+        dim = mtx_t.shape[-1]
+        if diff_var is None:
+            if dim == 1:
+                du = vec[:, [1]] - vec[:, [0]]
+            elif dim == 2:
+                du = vec[:, [1, 3]] - vec[:, [0, 2]]
+            elif dim == 3:
+                du = vec[:, [1, 3, 5]] - vec[:, [0, 2, 4]]
+
+            dx = nm.matmul(mtx_t.transpose((0, 2, 1)), du[..., None])[:, 0, :]
+            Fe = nm.zeros((2 * dim,), dtype=nm.float64)
+            Fe[0] = -1
+            Fe[1] = 1
+            out[...] = (Fe * dx)[:, None, :, None] * mat
+
+            if mtx_t is not None:
+                membranes.transform_asm_vectors(out, mtx_t)
+
+        else:
+            Ke = nm.zeros((2 * dim, 2 * dim), dtype=nm.float64)
+            Ke[0, 0] = Ke[1, 1] = 1
+            Ke[1, 0] = Ke[0, 1] = -1
+            out[...] = Ke * mat
+
+            if mtx_t is not None:
+                membranes.transform_asm_matrices(out, mtx_t)
+
+        return 0
+
+    @staticmethod
+    def get_mtx_t_and_length(coors, dx=None):
+        from sfepy.linalg import norm_l2_along_axis as norm
+
+        dim = coors.shape[-1]
+        if dx is None:
+            dx = coors[:, 1, :] - coors[:, 0, :]
+
+        mtx_t = nm.zeros((coors.shape[0], dim, dim), dtype=nm.float64)
+        length = norm(dx)[:, None]
+        v1 = dx / length
+        mtx_t[:, :, 0] = v1
+
+        if dim == 1:
+            pass
+        elif dim == 2:
+            mtx_t[:, 0, 1] = -v1[:, 1]
+            mtx_t[:, 1, 1] = v1[:, 0]
+        elif dim == 3:
+            v2 = nm.zeros_like(v1)
+            for k in [0, 2, 1]:
+                v2_ = nm.zeros((1, dim), dtype=nm.float64)
+                v2_[0, k] = 1.
+                dot = nm.abs(nm.sum(v2_ * v1, axis=1))
+                v2[(1. - dot) > 1e-12] = v2_
+
+            v3 = nm.cross(v1, v2)
+            v2 = nm.cross(v3, v1)
+            v2 = v2 / norm(v2)[:, None]
+            v3 = v3 / norm(v3)[:, None]
+            mtx_t[:, :, 1] = v2
+            mtx_t[:, :, 2] = v3
+        else:
+            raise ValueError(f'unsupported space dimension {dim}!')
+
+        return mtx_t, length
+
+    def get_fargs(self, mat, virtual, state,
+                  mode=None, term_mode=None, diff_var=None, **kwargs):
+        from sfepy.discrete.variables import create_adof_conn
+
+        econn = virtual.field.get_econn('cell', self.region)
+        coors = virtual.field.get_coor()[econn]
+
+        mtx_t, length = self.get_mtx_t_and_length(coors)
+
+        mat = mat / length[..., None, None]
+
+        if diff_var is None:
+            _, _, _, _, n_c = self.get_data_shape(virtual)
+            adc = create_adof_conn(nm.arange(state.n_dof, dtype=nm.int32),
+                                   econn, n_c, 0)
+
+            return mat, state()[adc], mtx_t, length, diff_var
+        else:
+            return mat, None, mtx_t, length, diff_var
+
+
+class LinearTrussInternalForceTerm(Term):
+    r"""
+    Evaluate internal force in the element direction. To be used with
+    'el_avg' or 'qp' evaluation modes which give the same results.
+    The material parameter :math:`EA` is equal to
+    Young modulus times element coss-section. The internal force
+    is given by :math:`F^{(i)} = -F^{(j)} = EA / l (U^{(j)} - U^{(i)})`,
+    where :math:`l` is the element length and :math:`U`, :math:`F` are
+    the nodal displacements and the nodal forces in the element direction.
+
+    :Definition:
+
+    .. math::
+        F = EA / l (U^{(j)} - U^{(i)})\\
+        \quad \forall \mbox{ elements } T_K^{i,j}\\
+        \mbox{ in a region connecting nodes } i, j
+
+    :Arguments:
+        - material : :math:`EA`
+        - parameter : :math:`\ul{w}`
+    """
+    name = 'ev_lin_truss_force'
+    arg_types = ('material', 'parameter')
+    arg_shapes = {'material': '1, 1', 'parameter': 'D'}
+    integration_order = 0
+    geometries = ['1_2', '2_1_2', '3_1_2']
+
+    @staticmethod
+    def function(out, mat, vec, mtx_t):
+        dim = mtx_t.shape[-1]
+        if dim == 1:
+            du = vec[:, [1]] - vec[:, [0]]
+        elif dim == 2:
+            du = vec[:, [1, 3]] - vec[:, [0, 2]]
+        elif dim == 3:
+            du = vec[:, [1, 3, 5]] - vec[:, [0, 2, 4]]
+
+        dx = nm.matmul(mtx_t.transpose((0, 2, 1)), du[..., None])[:, 0, :]
+        out[...] = dx[:, None, None, :] * mat
+
+        return 0
+
+    def get_fargs(self, mat, parameter,
+                  mode=None, term_mode=None, diff_var=None, **kwargs):
+        from sfepy.discrete.variables import create_adof_conn
+
+        econn = parameter.field.get_econn('cell', self.region)
+        coors = parameter.field.get_coor()[econn]
+
+        mtx_t, length = LinearTrussTerm.get_mtx_t_and_length(coors)
+
+        mat = mat / length[..., None, None]
+
+        _, _, _, _, n_c = self.get_data_shape(parameter)
+        adc = create_adof_conn(nm.arange(parameter.n_dof, dtype=nm.int32),
+                               econn, n_c, 0)
+
+        return mat, parameter()[adc], mtx_t
+
+    def get_eval_shape(self, mat, parameter,
+                       mode=None, term_mode=None, diff_var=None, **kwargs):
+        n_el, _, _, _, _ = self.get_data_shape(parameter)
+
+        return (n_el, 1, 1, 1), parameter.dtype
+
+
+class LinearDSpringTerm(LinearTrussTerm):
+    r"""
+    Linear spring element with the stiffness transformed into
+    the element direction.
+
+    :Definition:
+
+    .. math::
+        f^{(i)}_k = -f^{(j)}_k = K_{kl} (u^{(j)}_l - u^{(i)}_l)\\
+        \quad \forall \mbox{ elements } T_K^{i,j}\\
+        \mbox{ in a region connecting nodes } i, j
+
+    :Arguments:
+        - opt_material : :math:`\ul{d}`
+        - material : :math:`\ul{k}`
+        - virtual: :math:`\ul{v}`
+        - state: :math:`\ul{u}`
+
+    Stiffness matrix
+    :math:`\ul{K} = \ul{T(\ul{d})}^T \ul{K(\ul{k})} \ul{T(\ul{d})}`
+    is defined by 6 components
+    :math:`\ul{k} = [k_{u1}, k_{u2}, k_{u3}, k_{r1}, k_{r2}, k_{r3}]` in 3D
+    and by 3 components :math:`\ul{k} = [k_{u1}, k_{u2}, k_{r1}]`,
+    where :math:`k_{ui}` is the stiffness for the displacement DOF
+    and :math:`r_{ui}` is for the rotational DOF. Note that the components of
+    :math:`\ul{k}` are in the local coordinates system specified by a given
+    direction :math:`\ul{d}` or by the vector
+    :math:`\ul{d} = \ul{x}^{(j)} - \ul{x}^{(i)}` for non-coincidental end nodes.
+    The stiffness parameter :math:`\ul{K}` can also be defined as a 6x6 matrix
+    in 3D or a 3x3 matrix in 2D.
+    """
+    name = 'dw_lin_dspring'
+    arg_types = ('opt_material', 'material', 'virtual', 'state')
+    arg_shapes = [{'opt_material': 'D, 1', 'material': 'D, 1',
+                   'virtual': ('D', 'state'), 'state': 'D'},
+                  {'material': 'D, D'}, {'opt_material': None}]
+
+    integration_order = 0
+    geometries = ['1_2', '2_1_2', '3_1_2']
+
+    @staticmethod
+    def function(out, mat, vec, mtx_t, diff_var):
+        nel, _, dim = mtx_t.shape
+        ndof = mat.shape[2]
+        ntr = 2 * dim
+        ke = nm.zeros((nel, 2 * ndof, 2 * ndof), dtype=nm.float64)
+        for k in range(ndof):
+            for j in range(ndof):
+                ke[:, 2*k, 2*j] = ke[:, 2*k + 1, 2*j + 1] = mat[:, 0, k, j,]
+                ke[:, 2*k + 1, 2*j] = ke[:, 2*k, 2*j + 1] = -mat[:, 0, k, j]
+
+        if diff_var is None:
+            trans_vec = membranes.transform_asm_vectors
+            vec_loc = vec.copy()[..., None]
+            trans_vec(vec_loc[:, None, :ntr ,:], mtx_t.transpose((0, 2, 1)))
+            fe = dot_sequences(ke, vec_loc)
+            out[...] = fe[:, None, ...]
+            trans_vec(out[:, :, :ntr ,:], mtx_t)
+
+        else:
+            out[...] = ke[:, None, ...]
+            membranes.transform_asm_matrices(out[..., :ntr, :ntr], mtx_t)
+
+        return 0
+
+    def get_fargs(self, dvec, mat, virtual, state,
+                  mode=None, term_mode=None, diff_var=None, **kwargs):
+        from sfepy.discrete.variables import create_adof_conn
+
+        econn = virtual.field.get_econn('cell', self.region)
+        coors = virtual.field.get_coor()[econn]
+
+        if dvec is not None:
+            dvec = dvec[:, 0, :, 0]
+
+        mtx_t, _ = self.get_mtx_t_and_length(coors, dvec)
+
+        if mat.shape[-1] == 1:
+            n = mat.shape[-2]
+            mat_ = nm.zeros(mat.shape[:-1] + (mat.shape[-2],), dtype=mat.dtype)
+            ii = nm.arange(n)
+            mat_[..., ii, ii] = mat[..., 0]
+            mat = mat_
+
+        if diff_var is None:
+            _, _, _, _, n_c = self.get_data_shape(virtual)
+            adc = create_adof_conn(nm.arange(state.n_dof, dtype=nm.int32),
+                                   econn, n_c, 0)
+
+            return mat, state()[adc], mtx_t, diff_var
+        else:
+            return mat, None, mtx_t, diff_var
+
+
+class LinearDRotSpringTerm(LinearDSpringTerm):
+    __doc__ = LinearDSpringTerm.__doc__
+
+    name = 'dw_lin_dspring_rot'
+    arg_types = ('opt_material', 'material', 'virtual', 'state')
+    arg_shapes = [{'opt_material': 'D, 1', 'material': 'S, 1',
+                   'virtual': ('S', 'state'), 'state': 'S'},
+                  {'material': 'S, S'}, {'opt_material': None}]
