@@ -1,5 +1,5 @@
 # SfePy version
-__version__ = '2020.4'
+__version__ = '2025.3'
 
 # "Minimal" supported versions.
 NUMPY_MIN_VERSION = '1.3'
@@ -18,29 +18,29 @@ SCIKIT_UMFPACK_MIN_VERSION = '0.1'
 MESHIO_MIN_VERSION='4.0.0'
 PSUTIL_MIN_VERSION='1.0.0'
 PYVISTA_MIN_VERSION='0.23.0'
+OPT_EINSUM_MIN_VERSION='3.0.0'
+JAX_MIN_VERSION='0.2.0'
+DASK_MIN_VERSION='2.0.0'
+PRIMME_MIN_VERSION='3.0.0'
+OCT2PY_MIN_VERSION='5.6.0'
+PYTHON_MUMPS_MIN_VERSION='0.0.3'
+MUMPSPY_MIN_VERSION='1.2.0'
+IPCTK_MIN_VERSION='1.3.0'
+PYPARDISO_MIN_VERSION='0.4.6'
 
 CYTHON_MIN_VERSION = '0.14.1'
 
-def get_basic_info(version=__version__):
+def get_version(version=__version__):
     """
-    Return SfePy installation directory information. Append current git
-    commit hash to `version`.
+    Get sfepy version.
+
+    Appends current git commit hash to `version` if `is_release` is set to True
+    in the site configuration.
     """
     import os.path as op
-    from sfepy import Config
+    from sfepy.config import top_dir, site_config
 
-    # If installed, up_dir is '.', otherwise (in (git) source directory) '..'.
-    for up_dir in ['..', '.']:
-        top_dir = op.normpath(op.realpath(op.join(op.dirname(__file__),
-                                                  up_dir)))
-        aux = op.join(top_dir, 'README.rst')
-        if op.isfile(aux):
-            break
-    else:
-        raise RuntimeError('cannot determine SfePy top level directory!')
-
-    config = Config()
-    if not config.is_release():
+    if not site_config.is_release():
         # Append current git commit hash to __version__.
         master = op.join(top_dir, '.git/refs/heads/master')
         if op.isfile(master):
@@ -48,8 +48,6 @@ def get_basic_info(version=__version__):
             version += '+git.%s' % fd.readline().strip()
             fd.close()
 
-    in_source_tree = up_dir == '..'
+    return version
 
-    return version, top_dir, in_source_tree
-
-__version__, top_dir, in_source_tree = get_basic_info(__version__)
+__version__ = get_version(__version__)
